/*****************************************************************************
 *
 * FILE:	lock.c
 * DESCRIPTION:	Wyvern: file lock utility library
 * DATE:	Tue, May 27 2003
 * UPDATED:	Sat, Jun 10 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2003-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: lock.c,v 1.1 2006/06/09 17:26:31 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#if	HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif	/* HAVE_SYS_TYPES_H */
#include <errno.h>

/******************************************************************************
 *
 *	Glocal functions declaration
 *
 *****************************************************************************/
extern int	wu_get_locked_fd(void);
extern int	wu_read_lock(int);
extern int	wu_write_lock(int);
extern int	wu_append_lock(int);
extern int	wu_unlock(int);

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/

/******************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/

/******************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wu_get_locked_fd
 * Description:		get a locked file descriptor
 * Arguments:		none
 * Return Value:	successful: locked file descriptor, otherwise: -1
 */
int
wu_get_locked_fd(void)
{
  char		tempfile[] = "/tmp/.wuXXXXXX.lock";
  int		lockfd;
#ifndef	O_NOFOLLOW
  struct stat	sbuf;
#endif	/* O_NOFOLLOW */

#if	HAVE_MKSTEMP
  lockfd = mkstemp(tempfile);
#else
  if (mktemp(tempfile) == NULL) {
    return -1;
  }

#if	O_NOFOLLOW
  lockfd = open(tempfile, O_CREAT|O_EXCL|O_RDWR|O_EXLOCK|O_NOFOLLOW, 0600);
#else
  if (lstat(path, &sbuf) == 0 && S_ISLNK(sbuf.st_mode)) {
    return -1;
  }

  lockfd = open(tempfile, O_CREAT|O_EXCL|O_RDWR|O_EXLOCK, 0600);
#endif	/* O_NOFOLLOW */
#endif	/* HAVE_MKSTEMP */
  unlink(tempfile);

  return lockfd;
}

/*
 * Function Name:	wu_read_lock
 * Description:		shared file lock
 * Arguments:		file descriptor
 * Return Value:	successful: 0, otherwise: -1
 */
int
wu_read_lock(lockfd)
	int	lockfd;
{
  struct flock	lock;

  lock.l_start	= 0;
  lock.l_len	= 0;
  lock.l_pid	= getpid();
  lock.l_type	= F_RDLCK;
  lock.l_whence	= SEEK_SET;

  return fcntl(lockfd, F_SETLKW, &lock);
}

/*
 * Function Name:	wu_write_lock
 * Description:		exclusive file lock
 * Arguments:		file descriptor
 * Return Value:	successful: 0, otherwise: -1
 */
int
wu_write_lock(lockfd)
	int	lockfd;
{
  struct flock	lock;

  lock.l_start	= 0;
  lock.l_len	= 0;
  lock.l_pid	= getpid();
  lock.l_type	= F_WRLCK;
  lock.l_whence	= SEEK_SET;

  return fcntl(lockfd, F_SETLKW, &lock);
}

/*
 * Function Name:	wu_append_lock
 * Description:		exclusive file lock
 * Arguments:		file descriptor
 * Return Value:	successful: 0, otherwise: -1
 */
int
wu_append_lock(lockfd)
	int	lockfd;
{
  struct flock	lock;

  lock.l_start	= 0;
  lock.l_len	= 0;
  lock.l_pid	= getpid();
  lock.l_type	= F_WRLCK;
  lock.l_whence	= SEEK_END;

  return fcntl(lockfd, F_SETLKW, &lock);
}

/*
 * Function Name:	wu_unlock
 * Description:		unlock file lock
 * Arguments:		file descriptor
 * Return Value:	successful: 0, otherwise: -1
 */
int
wu_unlock(lockfd)
	int	lockfd;
{
  struct flock	unlock;

  unlock.l_start  = 0;
  unlock.l_len	  = 0;
  unlock.l_pid	  = getpid();
  unlock.l_type	  = F_UNLCK;
  unlock.l_whence = SEEK_SET;

  return fcntl(lockfd, F_SETLKW, &unlock);
}
