/*****************************************************************************
 *
 * FILE:	misc.c
 * DESCRIPTION:	WTL: miscellaneous functions
 * DATE:	Thu, Mar 27 2003
 * UPDATED:	Wed, Sep  5 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2003-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: misc.c,v 1.19 2007/09/04 17:09:03 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <stdarg.h>
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#if	HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif	/* HAVE_SYS_TYPES_H */
#if	HAVE_SYS_TIME_H
#include <sys/time.h>
#endif	/* HAVE_SYS_TIME_H */
#if	HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif	/* HAVE_SYS_RESOURCE_H */
#if	HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif	/* HAVE_SYS_STAT_H */
#if	HAVE_SYSLOG_H
#include <syslog.h>
#endif	/* HAVE_SYSLOG_H */
#if	HAVE_DIRENT_H
#include <dirent.h>
#else
#define dirent  direct
#if	HAVE_SYS_NDIR_H
#include <sys/ndir.h>
#endif	/* HAVE_SYS_NDIR_H */
#if	HAVE_SYS_DIR_H
#include <sys/dir.h>
#endif	/* HAVE_SYS_DIR_H */
#if	HAVE_NDIR_H
#include <ndir.h>
#endif	/* HAVE_NDIR_H */
#endif	/* HAVE_DIRENT_H */
#include <errno.h>
#if	ENABLE_SSL
#include "wt-ssl.h"
#else
#include "wt.h"
#endif	/* ENABLE_SSL */
#include "wt-sub.h"

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	PATH_DEVNULL	"/dev/null"
#if	HAVE_SYSCONF
#define	MAXFD		sysconf(_SC_OPEN_MAX)
#else
#define	MAXFD		(64)
#endif	/* HAVE_SYSCONF */

/* users/groups */
#define UID_ROOT	(0)	/* super user id in /etc/passwd */
#define GID_WHEEL	(0)	/* super user's group id in /etc/passwd */
#define UID_NOBODY	(65534)	/* nobody user id in /etc/passwd */
#define GID_NOGROUP	(65533)	/* nogroup id in /etc/passwd */

/******************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static bool	is_daemon = false;

/******************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wt_daemon
 * Description:		run in the background
 * Arguments:		no chdir, no close, syslog ident
 * Return Value:	successful: 0, otherwise: -1
 */
int
wt_daemon(nochdir, noclose, ident)
	int		nochdir;
	int		noclose;
	const char *	ident;
{
  struct rlimit	rl;

  /* detach the current process from TTY */
  switch (fork()) {
    case -1:	/* error */
      return -1;
    case 0:	/* child */
      break;
    default:	/* parent */
      _exit(0);
  }

  /* create new session */
  if (setsid() == -1) {
    return -1;
  }

  /* detach the process from the 1st child process */
  signal(SIGHUP, SIG_IGN);
  switch (fork()) {
    case -1:	/* error */
      exit(1);
    case 0:	/* child */
      break;
    default:	/* parent */
      _exit(0);
  }

  /* I'm a daemon */
  is_daemon = true;

  /* change directory */
  if (!nochdir) {
    (void)chdir("/");
  }

  /* clear the file mode mask */
  (void)umask(S_IWGRP|S_IWOTH);	/* 022 */

  /* close file descriptors */
  if (!noclose) {
    register int	fd;

    /* close all descriptors */
    for (fd = 0; fd < MAXFD; fd++) {
      close(fd);
    }

    fd = open(PATH_DEVNULL, O_RDWR, 0);
    if (fd != -1) {
      (void)dup2(fd, 0);
      (void)dup2(fd, 1);
      (void)dup2(fd, 2);
      if (fd > 2) {
	close(fd);
      }
    }
  }

  /* limit resource */
  if (getrlimit(RLIMIT_CORE, &rl) == 0) {
    rl.rlim_cur = rl.rlim_max = 0;	/* no core dump */
    if (setrlimit(RLIMIT_CORE, &rl) == -1) {
      return -1;
    }
  }

  /* open syslog */
  if (ident != NULL) {
    openlog(ident, LOG_PID, LOG_DAEMON);
  }

  return 0;
}

/*
 * Function Name:	wt_drop_privilege
 * Description:		drop the privilege of the process
 * Arguments:		uid, gid
 * Return Value:	successful: 0, error: -1, otherwise: 1
 */
int
wt_drop_privilege(uid, gid)
	uid_t	uid;
	gid_t	gid;
{
  /* drop privilege */
  if (uid == UID_ROOT) {
    uid = UID_NOBODY;
  }
  if (gid == GID_WHEEL) {
    gid = GID_NOGROUP;
  }

  /* change owner/group */
  if (getuid() == UID_ROOT) {
    struct passwd *	pwd;

    pwd = getpwuid(uid);
    if (pwd == NULL) {
      return -1;
    }

    if (initgroups(pwd->pw_name, (int)pwd->pw_gid) == -1) {
      return -1;
    }

    /* drop privilege */
    if (setgid(gid) == -1 || setuid(uid) == -1) {
      return -1;
    }

    return 0;
  }

  return 1;
}

/*
 * Function Name:	wt_store_pid
 * Description:		store the process id into pid file
 * Arguments:		pid file, uid, gid
 * Return Value:	successful: 0, error: -1
 */
int
wt_store_pid(pidfile, uid, gid)
	const char *	pidfile;
	uid_t		uid;
	gid_t		gid;
{
  FILE *	fout;
  int		fd;
  int		status;

  fd = open(pidfile, O_WRONLY|O_CREAT|O_EXCL, 0644);
  if (fd == -1) {
    return -1;
  }

  status = fchown(fd, uid, gid);
  fout	 = fdopen(fd, "w");
  if (fout == NULL) {
    unlink(pidfile);
    close(fd);
    return -1;
  }

  fprintf(fout, "%ld\n", (long)getpid());
  fclose(fout);

  return status;
}

/*
 * Function Name:	wt_printf
 * Description:		formatted output
 * Arguments:		format, arguments
 * Return Value:	successful: printed number, otherwise: -1
 */
int
#if	__STDC__
wt_printf(const char * fmt, ...)
#else
wt_printf(buf, size, fmt, va_alist)
	const char *	fmt;
	va_dcl
#endif	/* __STDC__ */
{
  va_list	ap;
  char		mesg[512];
  int		n;

#if	__STDC__
  va_start(ap, fmt);
#else
  va_start(ap);
#endif	/* __STDC__ */
  memset(mesg, 0, sizeof(mesg));
#if	HAVE_VSNPRINTF
  n = vsnprintf(mesg, sizeof(mesg), fmt, ap);
  va_end(ap);
#else
  n = vsprintf(mesg, fmt, ap);
  va_end(ap);
  if (n >= sizeof(mesg)) {
    return -1;
  }
#endif	/* HAVE_VSNPRINTF */

  if (is_daemon) {
    syslog(LOG_INFO, "%s", mesg);
  }
  else {
    fflush(stdout);
    fprintf(stderr, "%s\n", mesg);
  }

  return n;
}

/*****************************************************************************/

/*
 * Function Name:	wt_readdir
 * Description:		read directory entry
 * Arguments:		directory pointer, entry buffer, result pointer
 * Return Value:	successful: 0, otherwise: errno
 */
int
wt_readdir(dirp, entry, result)
	register DIR *		dirp;
	struct dirent *		entry;
	struct dirent **	result;
{
  struct dirent *	dp;
  int			saved_errno;

  saved_errno = errno;
  errno = 0;
  dp = readdir(dirp);
  if (errno != 0) {
    if (dp == NULL) {
      return errno;
    }
  }
  else {
    errno = saved_errno;
  }

  if (dp != NULL) {
    /* copy the result not to be overridden */
    *entry = *dp;
    if (sizeof(entry->d_name) <= 1) {
#if	HAVE_STRLCPY
      strlcpy(entry->d_name, dp->d_name, 255);
#else
      strncpy(entry->d_name, dp->d_name, strlen(dp->d_name));	/* XXX */
#endif	/* HAVE_STRLCPY */
    }
    *result = entry;
  }
  else {
    *result = NULL;
  }

  return 0;
}

