/*****************************************************************************
 *
 * FILE:	packet.c
 * DESCRIPTION:	Wyvern: packet manipulation library
 * DATE:	Mon, Feb 28 2005
 * UPDATED:	Wed, Sep  5 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2005-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2005-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: packet.c,v 1.4 2007/09/04 17:09:03 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#if	HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif	/* HAVE_SYS_TYPES_H */
#if	HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif	/* HAVE_SYS_SOCKET_H */
#if	HAVE_NETDB_H
#include <netdb.h>
#endif	/* HAVE_NETDB_H */
#if	HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif	/* HAVE_NETINET_IN_H */
#include <net/if.h>
#include <net/bpf.h>
#include <net/ethernet.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_var.h>
#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#include <netinet/tcpip.h>
#include <netinet/udp.h>
#include <netinet/udp_var.h>
#include <netinet/if_ether.h>
#if	HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif	/* HAVE_ARPA_INET_H */
#if	TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if	HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif	/* HAVE_SYS_TIME_H */
#endif	/* TIME_WITH_SYS_TIME */
#include <errno.h>
#include "wp.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * RFC 793
 * +--------+--------+--------+--------+
 * |           Source Address          |
 * +--------+--------+--------+--------+
 * |         Destination Address       |
 * +--------+--------+--------+--------+
 * |  zero  |  PTCL  | TCP/UDP Length  |
 * +--------+--------+--------+--------+
 */
struct pseudo_hdr {	/* TCP pseudo header for checksum */
  in_addr_t	saddr;
  in_addr_t	daddr;
  u_int8_t	zero;
  u_int8_t	protocol;
  u_int16_t	length;
};
#define	SIZE_ETHERHDR	(sizeof(struct ether_header))
#define	SIZE_IPHDR	(sizeof(struct ip))
#define	SIZE_TCPHDR	(sizeof(struct tcphdr))
#define	SIZE_UDPHDR	(sizeof(struct udphdr))
#define	SIZE_ICMPHDR	(sizeof(struct icmp))
#define	SIZE_BPFHDR	(sizeof(struct bpf_hdr))
#define	SIZE_PSEUDOHDR	(sizeof(struct pseudo_hdr))

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
int
wp_open_bpf(netif, capsize, prmisc, to_ms, pktfilter)
	const char *		netif;		/* network interface */
	unsigned int		capsize;	/* buffer size to capture */
	bool			prmisc;		/* promiscus: true|false */
	unsigned int		to_ms;		/* timeout (msec) */
	struct bpf_program *	pktfilter;	/* packet filter */
{
  if (netif != NULL) {
    int	fd = -1;
    int	n  = 0;

    do {
      char	s[sizeof("/dev/bpfXXX")];

      (void)snprintf(s, sizeof(s), "/dev/bpf%d", n++);
      fd = open(s, O_RDWR);
    } while (fd == -1 && (errno == EBUSY || errno == EPERM));

    if (fd != -1) {
      struct ifreq	ifreq;

      /* set buffer size to capture */
      if (capsize > 0) {
	unsigned int	bufsize;

	for (bufsize = capsize; bufsize != 0; bufsize >>= 1) {
	  if (ioctl(fd, BIOCSBLEN, &bufsize) == 0) {
	    break;
	  }
	}
	if (bufsize == 0) {
	  goto error;
	}
      }
      /* set network device */
      memset(&ifreq, 0, sizeof(ifreq));
      strlcpy(ifreq.ifr_name, netif, sizeof(ifreq.ifr_name));
      if ( ioctl(fd, BIOCSETIF, &ifreq) == -1) {
	goto error;
      }
      /* set packet filter */
      if (pktfilter != NULL) {
	if (ioctl(fd, BIOCSETF, pktfilter) == -1) {
	  goto error;
	}
      }
      /* set timeout */
      if (to_ms > 0) {
	struct timeval	timeout;

	timeout.tv_sec  = to_ms / 1000;
	timeout.tv_usec = (to_ms * 1000) % 1000000;
	if (ioctl(fd, BIOCSRTIMEOUT, &timeout) == -1) {
	  goto error;
	}
      }
      else {
	const int	imm = 1;

	if (ioctl(fd, BIOCIMMEDIATE, &imm) == -1) {
	  goto error;
	}
      }
      /* set promiscus mode */
      if (prmisc) {
       	if (ioctl(fd, BIOCPROMISC, NULL) == -1) {
	  goto error;
	}
      }
      if (ioctl(fd, BIOCFLUSH, 0) == 0) {
	return fd;
      }
error:
      close(fd);
    }
  }

  return -1;
}

/*****************************************************************************/

ssize_t
wp_build_tcp_packet(id, flags, seqno, ackno,
		    src_addr, src_port, dst_addr, dst_port, packet)
	u_int8_t	id;		/* identification */
	u_int8_t	flags;		/* tcp flags */
	u_int32_t	seqno;		/* tcp sequence number */
	u_int32_t	ackno;		/* tcp acknowledge number */
	in_addr_t	src_addr;	/* source address */
	in_port_t	src_port;	/* source port */
	in_addr_t	dst_addr;	/* destination address */
	in_port_t	dst_port;	/* destination port */
	u_int8_t **	packet;
{
  if (packet != NULL) {
    u_int8_t *	tcppacket;
    size_t	packetsize;
    size_t	tcpoptsize = 0;

    if ((flags & TH_SYN) == TH_SYN) {
      tcpoptsize = 12;	/* for timestamp option (see rFC1323) */
    }

    packetsize = SIZE_TCPHDR + tcpoptsize;
    tcppacket  = (u_int8_t *)malloc(SIZE_PSEUDOHDR + packetsize);
    if (tcppacket != NULL) {
      struct pseudo_hdr *	pseudo;
      struct tcphdr *		tcp;
      ssize_t			size;

      pseudo = (struct pseudo_hdr *)tcppacket;
      tcp    = (struct tcphdr *)(tcppacket + SIZE_PSEUDOHDR);

      memset(tcppacket, 0, SIZE_PSEUDOHDR + packetsize);
      /* build pseudo header */
      memcpy(&pseudo->saddr, &src_addr, sizeof(in_addr_t));
      memcpy(&pseudo->daddr, &dst_addr, sizeof(in_addr_t));
      pseudo->protocol = IPPROTO_TCP;
      pseudo->length   = htons(packetsize);

      /* build header */
      tcp->th_sport = src_port;
      tcp->th_dport = dst_port;
      tcp->th_seq   = htonl(seqno);
      tcp->th_ack   = htonl(ackno);
      tcp->th_x2    = 0;
      tcp->th_off   = (SIZE_TCPHDR >> 2) + (tcpoptsize >> 2);
      tcp->th_flags = flags;
      tcp->th_win   = htons(TTCP_CLIENT_SND_WND);
      tcp->th_urp   = 0;

      /* tcp timestamp option */
      if (tcpoptsize) {
	u_int8_t *	tstamp;
	time_t		now;

	tstamp = (u_int8_t *)(tcppacket + SIZE_PSEUDOHDR + SIZE_TCPHDR);
	tstamp[0] = 8;
	tstamp[1] = 10;	/* 10bytes = kind + length + T1 + T2 */
	time(&now);
	memcpy(tstamp + 2, &now, 4);
	memset(tstamp + 6, 0, 4);	/* zero clear */
	tstamp[10] = 1;	/* NOP */
	tstamp[11] = 0;	/* EOL */
      }

      /* calculate checksum */
      tcp->th_sum  = wp_cksum((u_int16_t *)tcppacket,
			      SIZE_PSEUDOHDR + packetsize);

      /* build ip packet */
      size = wp_build_ip_packet(id, IPPROTO_TCP, src_addr, dst_addr,
				  tcppacket + SIZE_PSEUDOHDR, packetsize,
				  packet);
      if (size > 0) {
	free(tcppacket);
      }

      return size;
    }
  }

  return -1;
}

ssize_t
wp_build_udp_packet(id, src_addr, src_port, dst_addr, dst_port, packet)
	u_int8_t	id;		/* identification */
	in_addr_t	src_addr;	/* source address */
	in_port_t	src_port;	/* source port */
	in_addr_t	dst_addr;	/* destination address */
	in_port_t	dst_port;	/* destination port */
	u_int8_t **	packet;
{
  if (packet != NULL) {
    u_int8_t *	udppacket;
    size_t	packetsize;

    packetsize = SIZE_UDPHDR;
    udppacket  = (u_int8_t *)malloc(SIZE_PSEUDOHDR + packetsize);
    if (udppacket != NULL) {
      struct pseudo_hdr *	pseudo;
      struct udphdr *		udp;
      ssize_t			size;

      pseudo = (struct pseudo_hdr *)udppacket;
      udp    = (struct udphdr *)(udppacket + SIZE_PSEUDOHDR);

      memset(udppacket, 0, SIZE_PSEUDOHDR + packetsize);
      /* build pseudo header */
      memcpy(&pseudo->saddr, &src_addr, sizeof(in_addr_t));
      memcpy(&pseudo->daddr, &dst_addr, sizeof(in_addr_t));
      pseudo->protocol = IPPROTO_UDP;
      pseudo->length   = htons(packetsize);

      /* build header */
      udp->uh_sport = src_port;
      udp->uh_dport = dst_port;
      udp->uh_ulen  = htons(packetsize);

      /* calculate checksum */
      udp->uh_sum   = wp_cksum((u_int16_t *)udppacket,
			      SIZE_PSEUDOHDR + packetsize);

      /* build ip packet */
      size = wp_build_ip_packet(id, IPPROTO_UDP, src_addr, dst_addr,
				  udppacket + SIZE_PSEUDOHDR, packetsize,
				  packet);
      if (size > 0) {
	free(udppacket);
      }

      return size;
    }
  }

  return -1;
}

ssize_t
wp_build_ip_packet(id, protocol, src_addr, dst_addr, data, datasize, packet)
	u_int8_t	id;		/* identification */
	u_int8_t	protocol;	/* protocol number */
	in_addr_t	src_addr;	/* source address */
	in_addr_t	dst_addr;	/* destination address */
	u_int8_t *	data;		/* paylaod */
	size_t		datasize;	/* payload size */
	u_int8_t **	packet;		/* returned */
{
  if (packet != NULL) {
    size_t	packetsize;

    packetsize = SIZE_IPHDR + datasize;
    *packet    = (u_int8_t *)malloc(packetsize);
    if (*packet != NULL) {
      struct ip *	ip;

      memset(*packet, 0, packetsize);
      ip = (struct ip *)(*packet);

      /* build header */
      ip->ip_v	 = IPVERSION;
      ip->ip_hl	 = SIZE_IPHDR >> 2;
      ip->ip_tos = 0;
      ip->ip_len = htons(packetsize);
      ip->ip_id	 = id;
      ip->ip_off = htons(IP_DF);
      ip->ip_ttl = IPDEFTTL;
      ip->ip_p	 = protocol;
      ip->ip_sum = 0;		/* zero clear to recompute */
      ip->ip_src = *(struct in_addr *)&src_addr;
      ip->ip_dst = *(struct in_addr *)&dst_addr;

      /* compute checksum */
      ip->ip_sum = wp_cksum((u_int16_t *)*packet, SIZE_IPHDR);

      /* copy payload */
      memcpy(*packet + SIZE_IPHDR, data, datasize);

      return packetsize;
    }
  }

  return -1;
}

ssize_t
wp_build_ether_frame(type, src_host, dst_host, data, datasize, frame)
	u_int16_t		type;		/* ether type */
	struct ether_addr *	src_host;	/* source host address */
	struct ether_addr *	dst_host;	/* destination host address */
	u_int8_t *		data;		/* paylaod */
	size_t			datasize;	/* payload size */
	u_int8_t **		frame;		/* returned */
{
  if (frame != NULL) {
    size_t	framesize;

    framesize = SIZE_ETHERHDR + datasize;
    *frame    = (u_int8_t *)malloc(framesize);
    if (*frame != NULL) {
      struct ether_header *	ether;

      memset(*frame, 0, framesize);
      ether = (struct ether_header *)(*frame);

      /* build header */
      ether->ether_type = htons(type);
      memcpy(ether->ether_shost, src_host->octet, ETHER_ADDR_LEN);
      memcpy(ether->ether_dhost, dst_host->octet, ETHER_ADDR_LEN);

      /* copy payload */
      memcpy(*frame + SIZE_ETHERHDR, data, datasize);

      return framesize;
    }
  }

  return -1;
}

u_int16_t
wp_cksum(addr, len)
	u_int16_t *	addr;
	size_t		len;
{
  int		nleft	= len;
  int		sum	= 0;
  u_int16_t *	w	= addr;
  u_int16_t	answer	= 0;
  union {
    u_int16_t	us;
    u_int8_t	uc[2];
  } last;

  /*
   * Our algorithm is simple, using a 32 bit accumulator (sum),
   * we add sequential 16 bit words to it, and at the end,
   * fold back all the carry bits from the top 16 bits into the lower 16 bits.
   */
  while (nleft > 1) {
    sum	  += *w++;
    nleft -= 2;
  }

  /* 4mop up an odd byte, if neccessary */
  if (nleft == 1) {
    last.uc[0] = *(u_int8_t *)w;
    last.uc[1] = 0;
    sum += last.us;
  }

  sum	 = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
  sum	+= (sum >> 16);			/* add carry */
  answer = ~sum;			/* truncate to 16 bits */

  return answer;
}

