/*****************************************************************************
 *
 * FILE:	sio.c
 * DESCRIPTION:	WTL: serial I/O functions
 * DATE:	Mon, May 12 2003
 * UPDATED:	Wed, Mar 30 2005
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MagickWorX.COM
 * URL:		http://www.MagickWorX.COM/
 * COPYRIGHT:	(c) 2003-2005 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2005 Kouichi ABE (WALL) <kouichi@MagickWorX.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: sio.c,v 1.5 2005/03/31 14:49:23 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#include <errno.h>
#if	HAVE_TERMIOS_H
#include <termios.h>
#endif	/* HAVE_TERMIOS_H */
#if	ENABLE_SSL
#include "wt-ssl.h"
#else
#include "wt.h"
#endif	/* ENABLE_SSL */
#include "wt-sub.h"

/******************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wt_netfd_initcom
 * Description:		initialize serial connection
 * Arguments:		device name, device speed
 * Return Value:	successful: netfd, otherwise: NULL
 */
wt_netfd_t *
wt_netfd_initcom(devname, devspeed)
	const char *	devname;
	unsigned int	devspeed;
{
  int			fd;
  struct termios	tty;
  wt_netfd_t *		nfd;

#if	O_NOFOLLOW
  fd = open(devname, O_RDWR|O_NONBLOCK|O_NOFOLLOW, 0600);
#else
  fd = open(devname, O_RDWR|O_NONBLOCK, 0600);
#endif	/* O_NOFOLLOW */
  if (fd < 0) {
    return NULL;
  }

  if (tcgetattr(fd, &tty) < 0) {
    close(fd);
    return NULL;
  }

  nfd = wt_netfd_fdopen(fd);
  if (nfd != NULL) {
    nfd->tty = tty;

    /* set communication speed */
    if (cfsetspeed(&tty, devspeed) < 0) {
      wt_netfd_close(nfd);
      return NULL;
    }
    /* set attributes */
    tty.c_cflag |= (tty.c_cflag & ~CSIZE) | CS8;	/* 8bit char */
    tty.c_cflag &= ~(PARENB|PARODD);			/* non-parity */
    tty.c_cflag &= ~CSTOPB;				/* no stop bit */
    tty.c_cflag &= ~CRTSCTS;				/* hardware flow off */
    tty.c_iflag &= ~(IXON|IXOFF|IXANY);			/* software flow off */
    /* set into raw, no echo mode */
    tty.c_iflag |= ICRNL;				/* map CR to NL */
    tty.c_lflag = 0;					/* no local mode */
    tty.c_oflag |= ONLCR;				/* map NL to CR-NL */
    tty.c_cflag |= (CLOCAL|CREAD);
    if (tcsetattr(fd, TCSANOW, &tty) < 0) {
      wt_netfd_close(nfd);
      return NULL;
    }

    return nfd;
  }

  return NULL;
}

/*
 * Function Name:	wt_netfd_com_close
 * Description:		close netfd for serial port
 * Arguments:		netfd
 * Return Value:	successful: 0, otherwise: -1
 */
int
wt_netfd_com_close(nfd)
	register wt_netfd_t *	nfd;
{
  tcflush(nfd->osfd, TCOFLUSH);
  tcsetattr(nfd->osfd, TCSANOW, &nfd->tty);
  return wt_netfd_close(nfd);
}

