/*****************************************************************************
 *
 * FILE:	wm.h
 * DESCRIPTION:	Wyvern: plugable module management library header file
 * DATE:	Sat, Sep 16 2000
 * UPDATED:	Sun, Aug 19 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2000-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2000-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wm.h,v 1.6 2007/08/19 05:57:22 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WM_H
#define	_WM_H

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WML_VERSION_NUMBER	0x10000020L
#define	WML_VERSION		"WML/1.0.0"
#define	WML_VERSION_TEXT	WML_VERSION " (2007/08/19)"
#define	WML_VERSION_TEXT_LONG \
	"Wyvern Module Management Library 1.0.0, Sun, Aug 19 2007"

#ifndef	WML_MODULE_MAGIC_NUMBER_MAJOR
#define	WML_MODULE_MAGIC_NUMBER_MAJOR	0x20060211
#endif	/* WML_MODULE_MAGIC_NUMBER_MAJOR */
#define	WML_MODULE_MAGIC_NUMBER_MINOR	0	/* 0...n */

/* Module structure */
typedef struct {
  const int	version;		/* API version */
  const char *	name;			/* module name */
  const char *	info;			/* description such as version etc.. */
  void *	handle;			/* module handle */
  void		(*init)(void *);	/* called when it's loaded */
  void		(*finish)(void);	/* called when it's unloaded */
#define	WML_MODULE_DLSYM_NAME	"wmModuleInfo"
  void		(*registration)(void);
} wmModule;

/*****************************************************************************
 *
 *	Global functions declaration
 *
 *****************************************************************************/
extern int		wm_load_module(const char *);
extern int		wm_unload_module(const char *);
extern void		wm_initialize_modules(void *);
extern void		wm_remove_modules(void);
extern wmModule *	wmModuleInfo(void);	/* You need implement this. */

#endif	/* _WM_H */
