/*****************************************************************************
 *
 * FILE:	wp.h
 * DESCRIPTION:	Wyvern: packet manipulation library header file
 * DATE:	Tue, Jul 20 2004
 * UPDATED:	Sun, Feb 26 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2004-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: wp.h,v 1.3 2006/02/25 22:34:50 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WP_H
#define	_WP_H

#include <sys/types.h>
#include <sys/time.h>
#include <net/bpf.h>
#include <net/ethernet.h>

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * Numeric release version identifier:
 * MNNFFPPS: major minor fix patch status
 * The status nibble has one of the values 0 for development,
 * 1 to e for betas 1 to 14, and f for release.
 * The patch level is exactly that.
 */
#define	WPL_VERSION_NUMBER	0x10001010L
#define	WPL_VERSION		"WPL/1.0.0"
#define	WPL_VERSION_TEXT	WPL_VERSION " (2006/02/26)"
#define	WPL_VERSION_TEXT_LONG \
	"Wyvern Packet Manipulation Library 1.0.0, Sun, Feb 26 2006"

#define	WPL_ETHER_ADDR_SIZE	(sizeof("ff:ff:ff:ff:ff:ff"))
#define	WPL_IP_ADDR_SIZE	(sizeof("255.255.255.255"))

#ifndef	_BOOL_T
#define	_BOOL_T
typedef enum {
  false = 0,
  true	= 1
} bool;
#endif	/* _BOOL_T */

/*****************************************************************************
 *
 *	Global functions declaration
 *
 *****************************************************************************/
/* beholder.c */
extern int		wp_open_bpf(const char *, unsigned int, bool,
				    unsigned int, struct bpf_program *);
extern ssize_t		wp_build_tcp_packet(u_int8_t, u_int8_t, u_int32_t,
					    u_int32_t, in_addr_t, in_port_t,
					    in_addr_t, in_port_t, u_int8_t **);
extern ssize_t		wp_build_udp_packet(u_int8_t, in_addr_t, in_port_t,
					    in_addr_t, in_port_t, u_int8_t **);
extern ssize_t		wp_build_ip_packet(u_int8_t, u_int8_t, in_addr_t,
					   in_addr_t, u_int8_t *, size_t,
					   u_int8_t **);
extern ssize_t		wp_build_ether_frame(u_int16_t, struct ether_addr *,
					     struct ether_addr *, u_int8_t *,
					     size_t, u_int8_t **);
extern u_int16_t	wp_cksum(u_int16_t *, size_t);

#endif	/* _WP_H */
