/*****************************************************************************
 *
 * FILE:	wt-sub.h
 * DESCRIPTION:	WTL: sub-header file
 * DATE:	Thu, Apr 17 2003
 * UPDATED:	Thu, Apr 28 2005
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2003-2005 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2003-2005 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *   ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *   FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 *
 * $Id: wt-sub.h,v 1.21 2005/05/31 17:59:22 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_WT_SUB_H
#define	_WT_SUB_H

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	CR	'\015'
#define	LF	'\012'
#define	CRLF	"\015\012"
#define	EOL	'\0'
#define	SPC	'\040'
#define	TAB	'\011'
#define	CR	'\015'

#ifndef	MAX
#define	MAX(a,b)	(((a)>(b))?(a):(b))
#endif	/* MAX */
#ifndef	MIN
#define	MIN(a,b)	(((a)<(b))?(a):(b))
#endif	/* MIN */

#if	EAGAIN != EWOULDBLOCK
#define	WT_IO_NOT_READY	((errno == EAGAIN) || (errno == EWOULDBLOCK))
#else
#define	WT_IO_NOT_READY	(errno == EAGAIN)
#endif

/*****************************************************************************/

/* clist operation */
#define	wt_clist_init(l,b)	\
	do {	\
		(l)->next = (l);	\
		(l)->prev = (l);	\
		(l)->body = (b);	\
	} while (0)
#define	wt_clist_insert_before(e,l)	\
	do {	\
		(e)->next = (l);	\
		(e)->prev = (l)->prev;	\
		(l)->prev->next = (e);	\
		(l)->prev = (e);	\
	} while (0)
#define	wt_clist_insert_after(e,l)	\
	do {	\
		(e)->next = (l)->next;	\
		(e)->prev = (l);	\
		(l)->next->prev = (e);	\
		(l)->next = (e);	\
	} while (0)
#define	wt_clist_next(e)	((e)->next)
#define	wt_clist_prev(e)	((e)->prev)
#define	wt_clist_append_link(e,l)	wt_clist_insert_before(e,l)
#define	wt_clist_insert_link(e,l)	wt_clist_insert_after(e,l)
#define	wt_clist_remove_link(e)	\
	do {	\
		(e)->prev->next = (e)->next;	\
		(e)->next->prev = (e)->prev;	\
		(e)->next = (e);	\
		(e)->prev = (e);	\
	} while (0)
#define	wt_clist_is_empty(l)	((l)->next == (l))
#define	wt_clis_head(l)		((l)->next)
#define	wt_clis_tail(l)		((l)->prev)


/* context operators */
#define	wt_context_save(thread) \
	( (thread)->ecode = errno, \
	  sigprocmask(SIG_SETMASK, &((thread)->sigs), NULL), \
	  sigsetjmp((thread)->jb, 1) )
#define	wt_context_restore(thread) \
	( errno = (thread)->ecode, \
	  siglongjmp((thread)->jb, 1) )
#define	wt_context_switch(thread) \
	if (wt_context_save(thread) == 0) \
		wt_vp_scheduler()

/*****************************************************************************/

typedef struct _wt_pollfd {
  int			fd;		/* file descriptor */
  unsigned short	events;		/* events to look for */
  unsigned short	revents;	/* events returned after select() */
} wt_pollfd_t;

typedef struct _wt_pollq {
  wt_clist_t	q;	/* link on linked list of polling queue */
  wt_thread_t	thread;	/* polling thread */
  wt_pollfd_t	poll;	/* poll descriptor */
  bool		on_q;	/* flag of state on queue */
} wt_pollq_t;

/*****************************************************************************/

#define	wt_current_thread	wt_thread_self()
#define	wt_thread_ptr(x)	((wt_thread_t)(x)->body)
#define	wt_pollq_ptr(x)		((wt_pollq_t *)(x)->body)

/* wrappers */
#define	wt_free(x)	\
	do { \
		if ((x) != NULL) { \
			free((x)); \
			(x) = NULL; \
		} \
	} while (0)

/******************************************************************************
 *
 *	Non-public Global Function Declaration
 *
 *****************************************************************************/
/* schd.c */
extern void	wt_vp_scheduler(void);
extern int	wt_poll(wt_pollfd_t *, wt_utime_t);
extern void	wt_append_run_q(wt_thread_t);
extern void	wt_remove_run_q(wt_thread_t);
extern void	wt_append_sleep_q(wt_thread_t, wt_utime_t);
extern void	wt_remove_sleep_q(wt_thread_t);
extern void	wt_append_zombie_q(wt_thread_t);
extern void	wt_remove_zombie_q(wt_thread_t);
extern void	wt_append_io_q(wt_pollq_t *, wt_utime_t);
extern void	wt_remove_io_q(wt_pollq_t *);
/* io.c */
extern int	wt_init_io(void);
/* proc.c */
extern int	wt_init_proc(void (*)(int));
extern int	vp_check_proc(void);

#endif	/* _WT_SUB_H */
