/*****************************************************************************
 *
 * FILE:	auth.h
 * DESCRIPTION:	Wyvern: authentication header
 * DATE:	Sat, Sep 18 1999
 * UPDATED:	Sat, Feb 11 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: auth.h,v 1.12 2006/02/12 21:07:06 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_auth_h
#define	_auth_h

#include <sys/types.h>
#include <sys/stat.h>
#include "wyvern.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
typedef struct {
  /* Programmer must need to set this function. */
  char *	(*get_password)(const char *, const char *);
  /*
   * Programmer can override the following functions in order to manage session
   * with the digest-opaque string.
   */
  char *	(*make_opaque)(Message *);
  int		(*check_password)(Message *, const char *, const char **,
				  size_t);
} AuthProc;

#define	WA_AUTH_TABLE_SIZE	(7)

/* Basic Authentication */
#define	WA_BASIC_ARGS_SIZE	(2)
#define	WA_BASIC_USERNAME	(0)
#define	WA_BASIC_PASSWORD	(1)

/* Digest Authentication */
#define	WA_DIGEST_ARGS_SIZE	(10)
#define	WA_DIGEST_USERNAME	(0)	// username
#define	WA_DIGEST_REALM		(1)	// realm
#define	WA_DIGEST_NONCE		(2)	// nonce
#define	WA_DIGEST_URI		(3)	// digest-uri
#define	WA_DIGEST_RESPONSE	(4)	// response
#define	WA_DIGEST_ALGORITHM	(5)	// algorithm
#define	WA_DIGEST_CNONCE	(6)	// cnonce
#define	WA_DIGEST_OPAQUE	(7)	// opaque
#define	WA_DIGEST_QOP		(8)	// message-qop
#define	WA_DIGEST_NC		(9)	// nonce-count

#define	WA_HASHLEN		(16)
typedef char		HASH[WA_HASHLEN];
#define	WA_HASHHEXLEN	(32)
typedef char		HASHHEX[WA_HASHHEXLEN+1];

/****************************************************************************/

extern int	wauth_digest_calc_ha1(const char *, const char *, const char *,
				      const char *, const char *, const char *,
				      HASHHEX);
extern int	wauth_digest_calc_response(HASHHEX, const char *, const char *,
					   const char *, const char *,
					   const char *, const char *, HASHHEX,
					   HASHHEX);
extern int	wauth_checker(Cache *, Message *, const char *);
extern void	wauth_cleaner(Cache *);
extern void	wauth_set_subr(AuthProc *);

#endif	/* _auth_h */
