/*****************************************************************************
 *
 * FILE:	mod_info.c
 * DESCRIPTION:	Wyvern: server information module
 * DATE:	Mon, Oct 11 2004
 * UPDATED:	Sat, Aug 18 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2004-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_info.c,v 1.19 2007/08/19 06:40:28 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <sys/mman.h>
#if	HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif	/* HAVE_SYS_PARAM_H */
#if	HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif	/* HAVE_SYS_TYPES_H */
#if	TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if	HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif	/* HAVE_SYS_TIME_H */
#endif	/* TIME_WITH_SYS_TIME */
#include <sys/sysctl.h>
/* df */
#include <sys/ucred.h>
#if	HAVE_SYS_MOUNT_H
#include <sys/mount.h>
#endif	/* HAVE_SYS_MOUNT_H */
/* ps */
#if	HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif	/* HAVE_SYS_STAT_H */
#include <sys/proc.h>
#include <sys/user.h>
#include <dirent.h>
#include <pwd.h>
/* vm */
#include <sys/vmmeter.h>
#if	HAVE_VM_VM_PARAM_H
#include <vm/vm_param.h>
#endif	/* HAVE_VM_VM_PARAM_H */
/* nsi */
#if	HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif	/* HAVE_SYS_SOCKET_H */
#if	HAVE_NET_ETHERNET_H
#include <net/ethernet.h>
#endif	/* HAVE_NET_ETHERNET_H */
#if	HAVE_NET_ETHERTYPES_H
#include <net/ethertypes.h>
#endif	/* HAVE_NET_ETHERTYPES_H */
#include <net/if.h>
#include <net/if_dl.h>
/* nsr */
#include <net/route.h>
#include <errno.h>
#include "wyvern.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	WM_MODULE_ID		"mod_info"

#define	MAGIC_URI_SYSTEM_INFO	"/system-info"
#define	MAGIC_TYPE_SYSTEM_INFO	"httpd/x-system-info"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	wm_info_uri_translater(Message *);
static int	wm_info_request_handler(Message *);
static void	make_content(Message *, FILE *);
static void	info_uptime(FILE *);
static void	info_df(FILE *);
static void	info_df_opt(struct statfs *, FILE *);
static void	info_ps(FILE *);
static void	info_vm(FILE *);
static void	info_nsi(FILE *);
static void	info_nsr(FILE *);
static void	escape_tag(char *, FILE *);
static void	wm_info_register_hooks(void);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
static int
wm_info_uri_translater(msg)
	register Message *	msg;
{
  DBG(fprintf(stderr, "[wm_info_uri_translater] uri = %s\n", msg->request.uri);)
  if (ws_strcmp(msg->request.uri, MAGIC_URI_SYSTEM_INFO) == 0) {
    msg->file.path = NULL;
    msg->file.mime = MAGIC_TYPE_SYSTEM_INFO;
    return WM_RET_OK;
  }

  return WM_RET_DECLINED;	/* continue to translate */
}

static int
wm_info_request_handler(msg)
	register Message *	msg;
{
  DBG(fprintf(stderr, "[wm_info_request_handler] mime = %s\n", msg->file.mime);)
  if (ws_strcmp(msg->file.mime, MAGIC_TYPE_SYSTEM_INFO) == 0) {
    int			fd;
    register FILE *	fout;
    struct stat		sbuf;

    /* initialize */
    msg->response.status     = STATUS_INTERNAL_SERVER_ERROR;
    msg->file.status.st_size = 0L;

    /* Open temporary file */
    fd = wu_mkstemp(httpd->tmpDir);
    if (fd == -1) {
      wv_syslog(WLL_LOG_ERR, "[%s] failed to create temporary file",
		WM_MODULE_ID);
      return WM_RET_ERROR;
    }

    /* Open output file */
    fout = fdopen(fd, "r+");
    if (fout == NULL) {
      wv_syslog(WLL_LOG_ERR, "[%s] failed to open file descriptor",
		WM_MODULE_ID);
      close(fd);
      return WM_RET_ERROR;
    }
    make_content(msg, fout);

    if (fstat(fd, &sbuf) == -1) {
      wv_syslog(WLL_LOG_ERR, "[%s] failed to get file status", WM_MODULE_ID);
      close(fd);
      return WM_RET_ERROR;
    }

    msg->file.nfd		= wt_netfd_fdopen(fd);
    msg->file.status.st_size	= sbuf.st_size;
    msg->file.mime		= MAGIC_TYPE_HTML;
    msg->response.status	= STATUS_REQUEST_OK;
    wt_netfd_seek(msg->file.nfd, (off_t)0L, WT_SEEK_SET);

    return WM_RET_OK;
  }

  return WM_RET_DECLINED;	/* continue to handle request */
}

static void
make_content(msg, fout)
	register Message *	msg;
	register FILE *		fout;
{
  /* make HTML text */
  fputs(
    "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n"
    "<html lang=\"ja\">\n"
    "<head>\n"
    "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=EUC-JP\">\n"
    "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n"
    "<style type=\"text/css\">\n"
    "<!--\n"
    "h1 { color: blue; text-align: center }\n"
    "span { color: #ff0033 }\n"
    "table { width: 100%; background-color: #bec3d2; border: 1px solid #dddddd }\n"
    "tr.none { background-color: transparent }\n"
    "tr { background-color: #f0f1f5 }\n"
    "th { background-color: transparent; text-align: left }\n"
    "td { color: #000033 }\n"
    "-->\n"
    "</style>\n"
    "<title>Information from Wyvern</title>\n"
    "</head>\n"
    "<body lang=\"ja\">\n"
    "<h1>System Information</h1>\n",
    fout);
  info_uptime(fout);
  if (ws_strcmp(msg->request.query_string, "disk") == 0) {
    info_df(fout);
  }
  else if (ws_strcmp(msg->request.query_string, "process") == 0) {
    info_ps(fout);
  }
  else if (ws_strcmp(msg->request.query_string, "memory") == 0) {
    info_vm(fout);
  }
  else if (ws_strcmp(msg->request.query_string, "interface") == 0) {
    info_nsi(fout);
  }
  else if (ws_strcmp(msg->request.query_string, "routing") == 0) {
    info_nsr(fout);
  }
  else {
    info_df(fout);
    info_ps(fout);
    info_vm(fout);
    info_nsi(fout);
    info_nsr(fout);
  }
  /* footer */
  fprintf(fout,
	"<address>"
	"<a href=\"%s\">%s</a> Server at %s Port %d"
	"</address>\n"
	"</body>\n"
	"</html>\n",
	SERVER_ADDRESS, SERVER_VERSION,
	msg->vhost->serverName, msg->soci.port);
  fflush(fout);
}

static void
info_uptime(fout)
	register FILE *	fout;
{
  int			mib[2];
  size_t		size;
  struct timeval	boottime;

  mib[0] = CTL_KERN;
  mib[1] = KERN_BOOTTIME;
  size	 = sizeof(boottime);
  if (sysctl(mib, 2, &boottime, &size, NULL, 0) != -1 &&
      boottime.tv_sec != 0) {
    time_t	now;
    struct tm *	tp;
    time_t	uptime;
    int		days, hours, mins, secs;
    double	avenrun[3];

    time(&now);

    fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Uptime Info\">\n"
	  "<thead>"
	  "<tr class=\"none\">"
	  "<th>Date</th>"
	  "<th>Uptime</th>"
	  "<th>CPU Load Averages</th>"
	  "</tr>\n"
	  "</thead>\n<tbody>\n<tr>", fout);
    tp = localtime(&now);
    fprintf(fout, "<td>%4d/%02d/%02d %02d:%02d:%02d</td><td>",
	    tp->tm_year + 1900,
	    tp->tm_mon + 1,
	    tp->tm_mday,
	    tp->tm_hour,
	    tp->tm_min,
	    tp->tm_sec);

    uptime = now - boottime.tv_sec;
    if (uptime > 60) {
      uptime += 30;
    }
    days    = uptime / 86400;
    uptime %= 86400;
    hours   = uptime / 3600;
    uptime %= 3600;
    mins    = uptime / 60;
    secs    = uptime % 60;
    if (days > 0) {
      fprintf(fout, "%d day%s,", days, days > 1 ? "s" : "");
    }
    if (hours > 0 && mins > 0) {
      fprintf(fout, "%2d:%02d", hours, mins);
    }
    else if (hours > 0) {
      fprintf(fout, "%d hr%s", hours, hours > 1 ? "s" : "");
    }
    else if (mins > 0) {
      fprintf(fout, "%d min%s", mins, mins > 1 ? "s" : "");
    }
    else {
      fprintf(fout, "%d sec%s", secs, secs > 1 ? "s" : "");
    }
    fputs("</td>", fout);

    if (getloadavg(avenrun, sizeof(avenrun) / sizeof(avenrun[0])) != -1) {
      unsigned int	i;

      fputs("<td>", fout);
      for (i = 0; i < (sizeof(avenrun) / sizeof(avenrun[0])); i++) {
	if (i > 0) {
	  fputs(",", fout);
	}
	fprintf(fout, " %.2f", avenrun[i]);
      }
      fputs("</td></tr>\n</tbody>\n</table>\n<hr>\n", fout);
    }
  }
}

static void
info_df(fout)
	register FILE *	fout;
{
  struct statfs *	mntbuf;
  int			n;

  n = getmntinfo(&mntbuf, MNT_NOWAIT);
  if (n > 0) {
    register int	i;

    fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Disk Info\">\n"
	  "<thead>"
	  "<tr class=\"none\">"
	  "<th>Filesystem</th>"
	  "<th>1K-blocks</th>"
	  "<th>Used</th>"
	  "<th>Avail</th>"
	  "<th>Mounted on</th>"
	  "<th>Options</th>"
	  "<th>Capacity</th>"
	  "</tr>\n"
	  "</thead>\n<tbody>\n", fout);
    for (i = 0; i < n; i++) {
      struct statfs *	sfp = &mntbuf[i];

      fputs("<tr><td>", fout);
      escape_tag(sfp->f_mntfromname, fout);
      fprintf(fout,
	      "</td>"
	      "<td>%ld</td>"
	      "<td>%ld</td>"
	      "<td>%ld</td>"
	      "<td>%s</td>",
	      (long)(sfp->f_blocks * (sfp->f_bsize / 1024)),
	      (long)((sfp->f_blocks - sfp->f_bfree) * (sfp->f_bsize / 1024)),
	      (long)sfp->f_bavail, sfp->f_mntonname);
      info_df_opt(sfp, fout);
      fputs("</tr>\n", fout);
    }
    fputs("</tbody>\n</table>\n<hr>\n", fout);
  }
}

static void
info_df_opt(sfp, fout)
	struct statfs *	sfp;
	register FILE *	fout;
{
  int		flags = sfp->f_flags;
  double	total = (double)sfp->f_blocks;
  double	used  = (double)(sfp->f_blocks - sfp->f_bavail);

  fprintf(fout, "<td>%s", sfp->f_fstypename);

  /* internal operation */
  if (flags & MNT_LOCAL)	{ fputs(", local", fout); }
  if (flags & MNT_QUOTA)	{ fputs(", quota", fout); }
  if (flags & MNT_ROOTFS)	{ fputs(", rootfs", fout); }
#if	MNT_USER
  if (flags & MNT_USER)		{ fputs(", user", fout); }
#endif	/* MNT_USER */
#if	MNT_IGNORE
  if (flags & MNT_IGNORE)	{ fputs(", ignore", fout); }
#endif	/* MNT_IGNORE */
  /* specifiable */
  if (flags & MNT_RDONLY)	{ fputs(", read-only", fout); }
  if (flags & MNT_SYNCHRONOUS)	{ fputs(", synchronous", fout); }
  if (flags & MNT_NOEXEC)	{ fputs(", noexec", fout); }
  if (flags & MNT_NOSUID)	{ fputs(", nosuid", fout); }
  if (flags & MNT_NODEV)	{ fputs(", nodev", fout); }
  if (flags & MNT_UNION)	{ fputs(", union", fout); }
  if (flags & MNT_ASYNC)	{ fputs(", asynchronous", fout); }
#if	MNT_SUIDDIR
  if (flags & MNT_SUIDDIR)	{ fputs(", suiddir", fout); }
#endif	/* MNT_SUIDDIR */
  if (flags & MNT_SOFTDEP)	{ fputs(", soft-updates", fout); }
#if	MNT_NOSYMFOLLOW
  if (flags & MNT_NOSYMFOLLOW)	{ fputs(", nosymfollow", fout); }
#endif	/* MNT_NOSYMFOLLOW */
  if (flags & MNT_NOATIME)	{ fputs(", noatime", fout); }
#if	MNT_NOCLUSTERR
  if (flags & MNT_NOCLUSTERR)	{ fputs(", noclusterr", fout); }
#endif	/* MNT_NOCLUSTERR */
#if	MNT_NOCLUSTERW
  if (flags & MNT_NOCLUSTERW)	{ fputs(", noclusterw", fout); }
#endif	/* MNT_NOCLUSTERW */

  fprintf(fout, "</td><td>%d%%</td>\n", (int)((used/total) * 100));
}

#if	ENABLE_SYSCTL_PS
#if __FreeBSD_version > 500113
static void 
info_ps(fout)
	register FILE *	fout;
{
  int		mib[4];
  size_t	len;

  mib[0] = CTL_KERN;
  mib[1] = KERN_PROC;
  mib[2] = KERN_PROC_ALL;
  if (sysctl(mib, 3, NULL, &len, NULL, 0) == 0) {
    char *	x;

    x = (char *)ws_malloc(len);
    if (sysctl(mib, 3, x, &len, NULL, 0) == 0) {
      int			num;
      struct kinfo_proc *	ki;
      register int		i;

      mib[2] = KERN_PROC_ARGS;
      num    = len / KINFO_PROC_SIZE;

      fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Process Info\">\n"
	    "<thead>"
	    "<tr class=\"none\">"
	    "<th>USERNAME</th>"
	    "<th>PID</th>"
	    "<th>PPID</th>"
	    "<th>PRI</th>"
	    "<th>NICE</th>"
	    "<th>TTY</th>"
	    "<th>STATE</th>"
	    "<th>WCHAN</th>"
	    "<th>COMMAND</th>"
	    "</tr>\n</thead>\n<tbody>\n", fout);
      ki = (struct kinfo_proc *)x;
      for (i = 0; i < num; i++) {
	int	plen;
	char	buf[MAXPATHLEN];

	mib[3] = ki[i].ki_pid;
	plen   = MAXPATHLEN;
	memset(buf, 0, sizeof(buf));
	if (sysctl(mib, 4, buf, &plen, NULL, 0) == 0) {
	  static const char *	p_stat[] = {
				  "??????",
				  "idle",
				  "run",
				  "sleep",
				  "stop",
				  "zombie",
				  "wait",
				  "locked"
				};
	  int	ma;
	  int	mi;
	  char	vty[5];

	  if (plen == 0) {
	    sprintf(buf, "(%s)", ki[i].ki_comm);
	  }
	  else {
	    int	j;

	    for (j = 0; j < plen - 1; j++) {
	      if (buf[j] == '\0') {
		buf[j] = ' ';
	      }
	    }
	  }
	  ma = major(ki[i].ki_tdev);
	  mi = minor(ki[i].ki_tdev);
	  switch (ma) {
	    case 255:
	      strlcpy(vty, "??", sizeof(vty));
	      break;
	    case 12:
	      if (mi != 255) {
		snprintf(vty, sizeof(vty), "v%d", mi);
		break;
	      }
	      /* FALLTHROUGH */
	    case 0:
	      strlcpy(vty, "con", sizeof(vty));
	      break;
	    case 5:
	      snprintf(vty, sizeof(vty), "p%d", mi);
	      break;
	  }
	  fprintf(fout,
		  "<tr>"
		  "<td>%s</td>"
		  "<td>%d</td>"
		  "<td>%d</td>"
		  "<td>%d</td>"
		  "<td>%d</td>"
		  "<td>%s</td>"
		  "<td>%s</td>"
		  "<td>%s</td>"
		  "<td>%s</td>"
		  "</tr>\n",
		  ki[i].ki_login,
		  ki[i].ki_pid,
		  ki[i].ki_ppid,
		  ki[i].ki_pri.pri_level, /* XXX check this */
		  ki[i].ki_nice,
		  vty,
		  p_stat[ki[i].ki_stat],
		  ki[i].ki_wmesg,
		  buf);
	}
      }
      fputs("</tbody>\n</table>\n<hr>\n", fout);
      free((void *)x);
    }
  }
}
#else
static void 
info_ps(fout)
	register FILE *	fout;
{
  int		mib[4];
  size_t	len;

  mib[0] = CTL_KERN;
  mib[1] = KERN_PROC;
  mib[2] = KERN_PROC_ALL;
  if (sysctl(mib, 3, NULL, &len, NULL, 0) == 0) {
    struct kinfo_proc *	x;

    x = (struct kinfo_proc *)ws_malloc(len);
    if (sysctl(mib, 3, x, &len, NULL, 0) == 0) {
      int	num;
      int	i;

      num = len / sizeof(struct kinfo_proc);
      fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Process Info\">\n"
	    "<thead>"
	    "<tr class=\"none\">"
	    "<th>USERNAME</th>"
	    "<th>PID</th>"
	    "<th>PPID</th>"
	    "<th>PRI</th>"
	    "<th>NICE</th>"
	    "<th>TTY</th>"
	    "<th>STATE</th>"
	    "<th>WCHAN</th>"
	    "<th>COMMAND</th>"
	    "</tr>\n</thead>\n<tbody>\n", fout);
      for (i = 0; i < num; i++) {
	static const char *	p_stat[] = {
				  "??????",
				  "idle",
				  "run",
				  "sleep",
				  "stop",
#if	defined(__NetBSD_Version__)
				  "zombie",
				  "dead",
				  "onproc"
#elif	defined(OpenBSD)
				  "zombie",
				  "dead"
#else
				  "zombie"
#endif
				};
	struct kinfo_proc *	u;
	char			vty[5];
	int			ma;
	int			mi;

	u  = (x + num - i - 1);
	ma = major(u->kp_eproc.e_tdev);
	mi = minor(u->kp_eproc.e_tdev);
	switch (ma) {
#if	defined(__NetBSD_Version__)
	  case 4095:
#else
	  case 255:
#endif
	    strlcpy(vty, "??", sizeof(vty));
	    break;
	  case 12:
	    if (mi != 255) {
	      snprintf(vty, sizeof(vty), "v%d", mi);
	      break;
	    }
	    /* FALLTHROUGH */
	  case 0:
	    strlcpy(vty, "con", sizeof(vty));
	    break;
	  case 5:
	    snprintf(vty, sizeof(vty), "p%d", mi);
	    break;
	}
	fprintf(fout,
		"<tr>"
		"<td>%s</td>"
		"<td>%d</td>"
		"<td>%d</td>"
		"<td>%d</td>"
		"<td>%d</td>"
		"<td>%s</td>"
		"<td>%s</td>"
		"<td>%s</td>"
		"<td>%s</td>"
		"</tr>\n",
		u->kp_eproc.e_login,
		u->kp_proc.p_pid,
		u->kp_eproc.e_ppid,
		u->kp_proc.p_priority,
		u->kp_proc.p_nice,
		vty,
		p_stat[(u_char)u->kp_proc.p_stat],
#if	defined(__NetBSD_Version__)
		u->kp_proc.p_stat == SONPROC ? "" : u->kp_eproc.e_wmesg,
#else
		u->kp_eproc.e_wmesg,
#endif
		u->kp_proc.p_comm);
      }
      fputs("</tbody>\n</table>\n<hr>\n", fout);
      free(x);
    }
  }
}
#endif
#else
static void
info_ps(fout)
	register FILE *	fout;
{
  register DIR *	dfd;

  dfd = opendir("/proc");
  if (dfd != NULL) {
    register struct dirent *	dp;

    fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Process Info\">\n"
	  "<thead>"
	  "<tr class=\"none\">"
	  "<th>USERNAME</th>"
	  "<th>PID</th>"
	  "<th>PPID</th>"
	  "<th>TTY</th>"
	  "<th>WCHAN</th>"
	  "<th>START</th>"
	  "<th>TIME</th>"
	  "<th>COMMAND</th>\n"
	  "</tr>\n"
	  "</thead>\n<tbody>\n",
	  fout);
    while ((dp = readdir(dfd)) != NULL) {
      char *	path;

      /* Skip '.' and '..' */
      if (dp->d_name[0]=='.') { continue; }
      /* Skip 'curproc' - it's us */
      if (dp->d_name[0]=='c') { continue; }

      asprintf(&path, "/proc/%s/status", dp->d_name);
      if (path != NULL) {
	FILE *	fin;

	fin = fopen(path, "r");
	if (fin != NULL) {
	  char	buf[256];

	  memset(buf, 0, sizeof(buf));
	  if (fgets(buf, sizeof(buf), fin) != NULL) {
#define	MAX_PROC_ARGS	(14)
#define	PROC_ARG_NAME	(0)	// command name
#define	PROC_ARG_PID	(1)	// process id
#define	PROC_ARG_PPID	(2)	// parent process id
#define	PROC_ARG_PGID	(3)	// process group id
#define	PROC_ARG_SID	(4)	// session id
#define	PROC_ARG_TTY	(5)	// tty
#define	PROC_ARG_FLAGS	(6)	// flags: ctty|sldr|noflags
#define	PROC_ARG_START	(7)	// start time
#define	PROC_ARG_USER	(8)	// user time
#define	PROC_ARG_SYSTEM	(9)	// system time
#define	PROC_ARG_WCHAN	(10)	// wait channel message
#define	PROC_ARG_EUID	(11)	// effective user id
#define	PROC_ARG_EGID	(12)	// effective group id
#define	PROC_ARG_CRED	(13)	// credentials
	    char **		ap;
	    char *		args[MAX_PROC_ARGS];
	    char *		p = buf;
	    char *		major;
	    char *		minor;
	    struct passwd *	pwd;
	    uid_t		uid;
	    struct tm *		tp;
	    time_t		up;
	    char *		s;

	    for (ap = args; (*ap = wu_strsep1(&p, " ")) != NULL; ) {
	      if (**ap != EOL) {
		if (++ap >= &args[MAX_PROC_ARGS]) {
		  break;
		}
	      }
	    }
	    major = strtok(args[PROC_ARG_TTY], ",");
	    minor = strtok(NULL, ",");
	    if (minor != NULL) {
	      if (ws_strcmp(minor, "-1") == 0)	    { minor = "?"; }
	      if (ws_strcmp(major, "-1") == 0)	    { major = "?"; }
	      else if (ws_strcmp(major, "12") == 0) { major = "v"; }
	      else if (ws_strcmp(major, "0")  == 0) { major = "con";
						      minor = "-"; }
	      else if (ws_strcmp(major, "5")  == 0) { major = "p"; }
	      else				    { major = "x"; }
	      if ((ws_strcmp(major, "v") == 0) &&
		  (ws_strcmp(minor, "255") == 0)) {
	       	major = "con";
		minor = "-";
	      }
	    }
	    uid = (uid_t)strtol(args[PROC_ARG_EUID], (char **)NULL, 10);
	    pwd = getpwuid(uid);
	    p	= strtok(args[PROC_ARG_START], ",");
	    up	= (time_t)strtol(p, (char **)NULL, 10);
	    tp	= localtime(&up);
	    p	= strtok(args[PROC_ARG_USER], ",");
	    up	= (time_t)strtol(p, (char **)NULL, 10);
	    p	= strtok(args[PROC_ARG_SYSTEM], ",");
	    up += (time_t)strtol(p, (char **)NULL, 10);
	    fprintf(fout,
		    "<tr>"
		    "<td>%s</td>"
		    "<td>%s</td>"
		    "<td>%s</td>"
		    "<td>%s%s</td>"
		    "<td>%s</td>"
		    "<td>%4d/%02d/%02d %02d:%02d</td>"
		    "<td>%3ld:%02ld</td><td>",
		    pwd->pw_name,
		    args[PROC_ARG_PID],
		    args[PROC_ARG_PPID],
		    major, minor == NULL ? "" : minor,
		    args[PROC_ARG_WCHAN],
		    tp->tm_year + 1900,
		    tp->tm_mon + 1,
		    tp->tm_mday,
		    tp->tm_hour,
		    tp->tm_min,
		    (long)(up / 60),
		    (long)(up % 60));

	    /* print command name */
	    if (*major == '?') { fputs("(", fout); }
	    s = args[PROC_ARG_NAME];
	    do {
	      p = strstr(s, "\\040");
	      if (p) {
		*p = '\0';
		p += 4;
		fprintf(fout, "%s ", s);
		s = p;
	      }
	    } while (p != NULL);
	    fprintf(fout, "%s", s);
	    if (*major == '?') { fputs(")", fout); }
	    fputs("</td></tr>\n", fout);
	  }
	  fclose(fin);
	}
	free(path);
      }
    }
    fputs("</tbody>\n</table>\n<hr>\n", fout);
    closedir(dfd);
  }
}
#endif	/* ENABLE_SYSCTL_PS */

static void
info_vm(fout)
	register FILE *	fout;
{
#define	PGTOK(x)	((x) * (unsigned int)pagesize >> 10)
  int			mib[2];
  size_t		len;
  struct vmtotal	v;

  mib[0] = CTL_VM;
  mib[1] = VM_METER;
  len	 = sizeof(struct vmtotal);
  if (sysctl(mib, 2, &v, &len, NULL, 0) == 0) {
    int		pagesize;

    pagesize = getpagesize();
    fprintf(fout,
	    "<table border=\"0\" cellspacing=\"0\" summary=\"Virtual Memory Info\">\n"
	    "<thead>\n"
	    "<tr class=\"none\">"
	    "<th colspan=\"2\">virtual memory</th>"
	    "<th colspan=\"2\">real memory</th>"
	    "<th colspan=\"2\">shared virtual memory</th>"
	    "<th colspan=\"2\">shared real memory</th>"
	    "<th>free memory pages</th>"
	    "</tr>\n"
	    "<tr class=\"none\">"
	    "<th>total</th><th>active</th>"
	    "<th>total</th><th>active</th>"
	    "<th>total</th><th>active</th>"
	    "<th>total</th><th>active</th>"
	    "<th></th>"
	    "</tr>\n</thead>\n<tbody>\n"
	    "<tr>"
	    "<td>%7ld</td><td>%7ld</td>"
	    "<td>%7ld</td><td>%7ld</td>"
	    "<td>%7ld</td><td>%7ld</td>"
	    "<td>%7ld</td><td>%7ld</td>"
	    "<td>%7ld</td>"
	    "</tr>\n</tbody>\n</table>\n<hr>\n",
	    (long)PGTOK(v.t_vm), (long)PGTOK(v.t_avm),
	    (long)PGTOK(v.t_rm), (long)PGTOK(v.t_arm),
	    (long)PGTOK(v.t_vmshr), (long)PGTOK(v.t_vmshr),
	    (long)PGTOK(v.t_rmshr), (long)PGTOK(v.t_rmshr),
	    (long)PGTOK(v.t_free));
  }
}

static void
info_nsi(fout)
	register FILE *	fout;
{
  int		mib[6];
  size_t	len;

  /* Get the interfaces table */
  mib[0] = CTL_NET;
  mib[1] = PF_ROUTE;
  mib[2] = 0;
  mib[3] = AF_LINK;
  mib[4] = NET_RT_IFLIST;
  mib[5] = 0;

  /* Estimate the size of table */
  if (sysctl(mib, 6, NULL, &len, NULL, 0) == 0) {
    char *	buf;

    buf = (char *)ws_malloc(len);
    /* Now get it. */
    if (sysctl(mib, 6, buf, &len, NULL, 0) == 0) {
      char *			end;
      char *			next;
      struct if_msghdr *	ifm;

      end = buf + len;
      fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Network Interfaces\">\n"
	    "<thead>\n"
	    "<tr class=\"none\">"
	    "<th>Name</th>"
	    "<th>Mtu</th>"
	    "<th>Network</th>"
	    "<th>Address</th>"
	    "<th>Ipkts</th>"
	    "<th>Ierrs</th>"
	    "<th>Opkts</th>"
	    "<th>Oerrs</th>"
	    "<th>Coll</th>"
	    "<th>Drops</th>"
	    "</tr>\n</thead>\n<tbody>\n", fout);

      /* scan the list and store base values */
      for (next = buf; next < end; next += ifm->ifm_msglen) {
	struct sockaddr_dl *	sdl;
	char			nic[20];

	ifm = (struct if_msghdr *)next;
	sdl = (struct sockaddr_dl *)(ifm + 1);
	if ((ifm->ifm_flags & IFF_UP) == 0) {
	  continue;	/* not up */
	}

	memset(nic, 0, sizeof(nic));
	memcpy(nic, sdl->sdl_data, sdl->sdl_nlen);
	fprintf(fout,
		"<tr>"
		"<td>%s</td><td>%lu</td><td>&lt;Link#%d&gt;</td><td>",
		nic, ifm->ifm_data.ifi_mtu, sdl->sdl_index);
	if (sdl->sdl_alen == 6) {
	  fprintf(fout, "%s",
		  ether_ntoa((struct ether_addr *)LLADDR(sdl)));
	}
	fprintf(fout,
		"</td>"
		"<td>%lu</td><td>%lu</td>"
		"<td>%lu</td><td>%lu</td>"
		"<td>%lu</td><td>%lu</td></tr>\n",
		ifm->ifm_data.ifi_ipackets,
		ifm->ifm_data.ifi_ierrors,
		ifm->ifm_data.ifi_opackets,
		ifm->ifm_data.ifi_oerrors,
		ifm->ifm_data.ifi_collisions,
		ifm->ifm_data.ifi_iqdrops);
      }
      fputs("</tbody>\n</table>\n<hr>\n", fout);
    }
    free(buf);
  }
}

static void
info_nsr(fout)
	register FILE *	fout;
{
/* alignment constraint for routing socket */
#define	ROUNDUP(a) \
	((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
#define NEXT_SA(ap) \
	ap=(struct sockaddr *)((caddr_t)ap+(ap->sa_len ? ROUNDUP(ap->sa_len) : sizeof(long)))
  int		mib[6];
  size_t	len;

  /* Get the routing table */
  mib[0] = CTL_NET;
  mib[1] = PF_ROUTE;
  mib[2] = 0;
  mib[3] = 0;
  mib[4] = NET_RT_DUMP;
  mib[5] = 0;

  /*Estimate the size of table */
  if (sysctl(mib, 6, NULL, &len, NULL, 0) == 0) {
    char *	buf;

    buf = (char *)ws_malloc(len);
    /* Now get it. */
    if (sysctl(mib, 6, buf, &len, NULL, 0) == 0) {
      char *			end;
      char *			next;
      struct rt_msghdr *	rtm;

      /* Now dump the routing table */
      fputs("<table border=\"0\" cellspacing=\"0\" summary=\"Network Routing\">\n"
	    "<thead>\n"
	    "<tr class=\"none\">"
	    "<th>Destination</th>"
	    "<th>Gateway</th>"
	    "<th>Flags</th>"
	    "<th>Use</th>"
	    "</tr>\n</thead>\n<tbody>\n", fout);
      end = buf + len;

      for (next = buf; next < end; next += rtm->rtm_msglen) {
	struct sockaddr *	sa;
	char			fbuf[50];

	rtm = (struct rt_msghdr *)next;
	sa  = (struct sockaddr *)(rtm + 1);

	memset(fbuf, 0, sizeof(fbuf));
	if (sa->sa_family == AF_INET6) { continue; }
	fputs("<tr>", fout);
	if (rtm->rtm_flags & RTF_HOST) {	/* host */
#if	NO_CLONED_ROUTE
	  if (rtm->rtm_flags & RTF_WASCLONED) {
	    if ((rtm->rtm_flags & RTF_LLINFO) == 0) {
	      continue;
	    }
	  }
#endif	/* NO_CLONED_ROUTE */
	  fprintf(fout, "<td>%s</td>", wt_sock_ntop(sa, false));
	}
	else if (rtm->rtm_flags & RTF_GATEWAY) {	/* gateway */
	  fprintf(fout, "<td>%s</td>", wt_sock_ntop(sa, false));
	}
	else if (rtm->rtm_flags & RTF_CLONING) {	/* network */
	  len	= strlcpy(fbuf, wt_sock_ntop(sa, false), sizeof(fbuf));
	  if (len < sizeof(fbuf)) {
	    struct sockaddr *	sa1;

	    sa1 = (struct sockaddr *)(ROUNDUP(sa->sa_len) + (caddr_t)sa);
	    NEXT_SA(sa1);
	    if (sa1->sa_family == 255) {
	      strlcat(fbuf, wt_sock_ntop(sa1, false), sizeof(fbuf));
	      sa = sa1;
	    }
	  }
	  fprintf(fout, "<td>%s</td>", fbuf);
	}
	else {
	  fprintf(fout, "<td>%d</td><td>%d</td></tr>\n",
		  rtm->rtm_addrs, rtm->rtm_flags);
	  continue;
	}
	NEXT_SA(sa);
	fprintf(fout, "<td>%s</td><td>", wt_sock_ntop(sa, false));
	if (rtm->rtm_flags & RTF_UP)		{ fputs("U", fout); }
	if (rtm->rtm_flags & RTF_GATEWAY)	{ fputs("G", fout); }
	if (rtm->rtm_flags & RTF_HOST)		{ fputs("H", fout); }
	if (rtm->rtm_flags & RTF_REJECT)	{ fputs("r", fout); }
	if (rtm->rtm_flags & RTF_DYNAMIC)	{ fputs("d", fout); }
	if (rtm->rtm_flags & RTF_CLONING)	{ fputs("C", fout); }
	if (rtm->rtm_flags & RTF_LLINFO)	{ fputs("L", fout); }
	if (rtm->rtm_flags & RTF_STATIC)	{ fputs("S", fout); }
#if	RTF_PRCLONING
	if (rtm->rtm_flags & RTF_PRCLONING)	{ fputs("c", fout); }
#endif	/* RTF_PRCLONING */
#if	RTF_WASCLONED
	if (rtm->rtm_flags & RTF_WASCLONED)	{ fputs("W", fout); }
#endif	/* RTF_WASCLONED */
#if	RTF_BROADCAST
	if (rtm->rtm_flags & RTF_BROADCAST)	{ fputs("b", fout); }
#endif	/* RTF_BROADCAST */
	fprintf(fout, "</td><td>%u</td></tr>\n", rtm->rtm_use);
      }
      fputs("</tbody>\n</table>\n<hr>\n", fout);
    }
    free(buf);
  }
}

static void
escape_tag(s, fout)
	register char *	s;	/* string */
	register FILE *	fout;
{
  register char *	c;

  for (c = s; *c != EOL; c++) {
    switch (*c) {
      case '<': fputs("&lt;", fout); break;
      case '>': fputs("&gt;", fout); break;
      default:  fputc(*c, fout);     break;
    }
  }
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_info_register_hooks(void)
{
  wm_hook_uri_translater(wm_info_uri_translater, HOOK_FIRST);
  wm_hook_request_handler(wm_info_request_handler, HOOK_FIRST, MAGIC_TYPE_SYSTEM_INFO);
}

static wmModule	mod_info = {
  WM_MAGIC_NUMBER_MAJOR,			/* version */
  WM_UNSECURE,					/* unsecure */
  WM_MODULE_ID,					/* name */
  "Wyvern Information Module version 1.0.8",	/* info */
  "Kouichi ABE (WALL)",				/* author */
  NULL,						/* depend */
  NULL,						/* handle */
  (void *)NULL,					/* init */
  (void *)NULL,					/* finish */
  (void *)wm_info_register_hooks		/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_info;
}
