/*****************************************************************************
 *
 * FILE:	mod_md5.c
 * DESCRIPTION:	Wyvern: MD5 module
 * DATE:	Tue, Aug 20 2002
 * UPDATED:	Wed, Mar  1 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2002-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_md5.c,v 1.18 2006/03/01 13:21:37 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <sys/mman.h>
#if	ENABLE_SSL
#include <openssl/md5.h>
#define	MD5Init		MD5_Init
#define	MD5Update	MD5_Update
#define	MD5Final	MD5_Final
#else
#include <md5.h>
#endif	/* ENABLE_SSL */
#include "wyvern.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_md5"

#ifndef	MD5_DIGEST_LENGTH
#define	MD5_DIGEST_LENGTH	(16)
#endif	/* MD5_DIGEST_LENGTH */

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	wm_set_md5(Message *);
static void	wm_md5_register_hooks(void);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_set_md5
 * Description:		set the message digest of file entity
 * Arguments:		message
 * Return Value:	successful: 0, otherwise: -1
 */
static int
wm_set_md5(msg)
	register Message *	msg;
{
  if (msg->response.status == STATUS_REQUEST_OK) {
    int		fd;
    char *	data;

    fd	 = wt_netfd_fileno(msg->file.nfd);
    data = mmap(NULL,
		(size_t)msg->file.status.st_size, 
		PROT_READ,
		MAP_PRIVATE,
		fd,
		(off_t)0);
    if (data != MAP_FAILED) {
      MD5_CTX		md5_ctx;
      unsigned char	md5_hex[MD5_DIGEST_LENGTH*2+1];

      memset(md5_hex, 0, sizeof(md5_hex));
      MD5Init(&md5_ctx);
      MD5Update(&md5_ctx, data, (size_t)msg->file.status.st_size);
#if	ENABLE_SSL
      do {
	register int		i;
	unsigned char		md5_bin[MD5_DIGEST_LENGTH];
	static const char	hex[] = "0123456789abcdef";

	memset(md5_hex, 0, sizeof(md5_hex));
	MD5Final(md5_bin, &md5_ctx);
	for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
	  md5_hex[i+i]   = hex[md5_bin[i] >> 4];
	  md5_hex[i+i+1] = hex[md5_bin[i] & 0x0f];
	}
      } while (0);
#else
      MD5End(&md5_ctx, md5_hex);
#endif	/* ENABLE_SSL */
      (void)lseek(fd, (off_t)0L, SEEK_SET);

      wa_put(msg->response.header, "Content-MD5", md5_hex);

      munmap(data, (size_t)msg->file.status.st_size);
    }
  }

  return WM_RET_OK;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_md5_register_hooks(void)
{
  wm_hook_fixups(wm_set_md5, HOOK_FIXUP_RESPONSE);
}

static wmModule	mod_md5 = {
  WM_MAGIC_NUMBER_MAJOR,	/* version */
  WM_SECURE,			/* secure */
  WM_MODULE_ID,			/* name */
  "MD5 Module version 1.1.3",	/* info */
  "Kouichi ABE (WALL)",		/* author */
  NULL,				/* depend */
  NULL,				/* handle */
  NULL,				/* init */
  NULL,				/* finish */
  (void *)wm_md5_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_md5;
}
