/*****************************************************************************
 *
 * FILE:	alias.c
 * DESCRIPTION:	Wyvern: register Redirect/Alias/ScriptAlias
 * DATE:	Sun, Sep 26 1999
 * UPDATED:	Sat, Aug 18 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: alias.c,v 1.15 2007/08/19 06:45:31 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include "wyvern.h"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static char *	lookup(const Alias *, const char *, alias_t);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wv_add_aliases
 * Description:		register a Redirect, Alias, or ScriptAlias into the
 *			aliasTable.
 * Arguments:		aliasTable, fakeName, realName, alias_type
 * Return Value:	none
 */
void
wv_add_aliases(aliasTable, fakeName, realName, type)
	Alias **	aliasTable;
	const char *	fakeName;
	const char *	realName;
	alias_t		type;
{
  if (lookup(*aliasTable, fakeName, type) == NULL) {
    Alias *	new;

    new		  = (Alias *)ws_malloc(sizeof(Alias));
    new->type	  = type;
    new->fakeName = ws_strdup(fakeName);
    new->fakeLen  = strlen(fakeName);
    new->realName = ws_strdup(realName);
    new->realLen  = strlen(realName);
    new->next	  = *aliasTable;
    *aliasTable	  = new;
  }
}

/*
 * Function Name:	lookup
 * Description:		lookup the Redirect, Alias, or ScriptAlias from the
 *			aliasTable.
 * Arguments:		aliasTable, fakeName, alias_type
 * Return Value:	realName
 */
static char *
lookup(aliasTable, fakeName, type)
	const Alias *	aliasTable;
	const char *	fakeName;
	alias_t		type;
{
  register Alias *	ap;

  for (ap = (Alias *)aliasTable; ap != NULL; ap = ap->next) {
    if (type == ap->type &&
	ws_strncmp(fakeName, ap->fakeName, ap->fakeLen) == 0) {
      return ap->realName;
    }
  }

  return NULL;
}

/*****************************************************************************/

/*
 * Function Name:	wv_alias_translater
 * Description:		translate an aliases of request URI
 * Arguments:		message
 * Return Value:	successful: 0, no translated: 1, otherwise: -1
 */
int
wv_alias_translater(msg)
	register Message *	msg;
{
  register Alias *	ap;

  /* Lookup ScriptAlias, Alias, or Redirect */
  for (ap = msg->vhost->aliasTable; ap != NULL; ap = ap->next) {
    if (ws_strncmp(msg->request.uri, ap->fakeName, ap->fakeLen) == 0 ||
	ws_strncmp(msg->request.uri, ap->fakeName, ap->fakeLen - 1) == 0) {
      register char *	urip;
      register int	n;

      switch (ap->type) {
	/* Redirection */
	case A_REDIRECT:
	  wa_put(msg->response.header, "Location", ap->realName);
	  msg->response.status = STATUS_MOVED_PERMANENTLY;
	  return WM_RET_OK;
	/* Alias */
	case A_ALIAS:
	  ws_asprintf(&msg->file.path, "%s%s",
		     ap->realName, msg->request.uri + ap->fakeLen);
	  return wv_complement_path(msg);
	/* Script Alias */
	case A_SCRIPT:
	  /* Set PATH-INFO */
	  urip = (char *)strchr(msg->request.uri + ap->fakeLen, '/');
	  if (urip != NULL) {
	    msg->request.path_info = ws_strdup(urip);
	    *urip = EOL;
	  }
	  n = ws_asprintf(&msg->file.path, "%s%s",
			 ap->realName, msg->request.uri + ap->fakeLen);
	  /* Directory listing for cgi-bin directory */
	  if (msg->file.path[n - 1] == '/') {
	    msg->response.status = STATUS_BAD_REQUEST;
	    return WM_RET_ERROR;
	  }
	  msg->file.mime = MAGIC_TYPE_CGI;
	  DBG(fprintf(stderr, "[translateAlias] %s\n", msg->file.path);)
	  return WM_RET_OK;
	/* Ignore */
	default:
	  break;
      }
    }
  }

  return WM_RET_DECLINED;	/* Continue to translate */
}
