/*****************************************************************************
 *
 * FILE:	date.c
 * DESCRIPTION:	Wyvern: time/date functions
 * DATE:	Wed, Dec  1 1999
 * UPDATED:	Sun, Aug 27 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: date.c,v 1.11 2006/08/27 07:30:30 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include "wyvern.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	DATE_BUFSIZE	(36)

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wv_convert_date
 * Description:		convert RFC822/1123 date format to the time_t value
 * Arguments:		HTTP-date string
 * Return Value:	time_t value
 */
time_t
wv_convert_date(date)
	const char *	date;
{
  struct tm	gmt;	/* according to the current timezone setting */
  char		month[10];
  char *	p;
  time_t	utc_date;

  /*
   * Some browser send the strange If-Modified-Since request field such as
   *	Tue, 23 Jan 2001 12:28:29 GMT; length=969
   *
   * I read RFC2616 and I couldn't understand the mean of "length"... 
   * So, I added the following code to solve the above problem.
   */
  p = strchr(date, ';');
  if (p != NULL) {
    *p = EOL;
  }

  if (strlen(date) > DATE_BUFSIZE) {
    return (time_t)0;
  }

  /*
   * Sun, 06 Nov 1994 08:49:37 GMT
   */
  if (sscanf(date, "%*s %02d %3s %4d %02d:%02d:%02d %*s",
		&(gmt.tm_mday), month, &(gmt.tm_year),
		&(gmt.tm_hour), &(gmt.tm_min), &(gmt.tm_sec)) == 6) {
    gmt.tm_year -= 1900;
  }
  /*
   * Sunday, 06-Nov-94 08:49:37 GMT
   */
  else if (sscanf(date, "%*s %02d-%3s-%2d %02d:%02d:%02d %*s",
		&(gmt.tm_mday), month, &(gmt.tm_year),
		&(gmt.tm_hour), &(gmt.tm_min), &(gmt.tm_sec)) == 6) {
  }
  /*
   * Sun Nov  6 08:49:37 1994 (ANSI C's asctime() format)
   */
  else if (sscanf(date, "%*s %3s %2d %02d:%02d:%02d %4d",
		month, &(gmt.tm_mday),
		&(gmt.tm_hour), &(gmt.tm_min), &(gmt.tm_sec),
		&(gmt.tm_year)) == 6) {
    gmt.tm_year -= 1900;
  }
  /*
   * Sun, Nov  6 1994 08:49:37 GMT
   */
  else if (sscanf(date, "%*s %3s %2d %4d %02d:%02d:%02d %*s",
		month, &(gmt.tm_mday), &(gmt.tm_year),
		&(gmt.tm_hour), &(gmt.tm_min), &(gmt.tm_sec)) == 6) {
    gmt.tm_year -= 1900;
  }
  /*
   * Unknown time format
   */
  else {
    return (time_t)0;
  }

  /*
   * convert string month to number
   */
  if	  (ws_strcasecmp(month, "Jan") == 0)	gmt.tm_mon = 0;
  else if (ws_strcasecmp(month, "Feb") == 0)	gmt.tm_mon = 1;
  else if (ws_strcasecmp(month, "Mar") == 0)	gmt.tm_mon = 2;
  else if (ws_strcasecmp(month, "Apr") == 0)	gmt.tm_mon = 3;
  else if (ws_strcasecmp(month, "May") == 0)	gmt.tm_mon = 4;
  else if (ws_strcasecmp(month, "Jun") == 0)	gmt.tm_mon = 5;
  else if (ws_strcasecmp(month, "Jul") == 0)	gmt.tm_mon = 6;
  else if (ws_strcasecmp(month, "Aug") == 0)	gmt.tm_mon = 7;
  else if (ws_strcasecmp(month, "Sep") == 0)	gmt.tm_mon = 8;
  else if (ws_strcasecmp(month, "Oct") == 0)	gmt.tm_mon = 9;
  else if (ws_strcasecmp(month, "Nov") == 0)	gmt.tm_mon = 10;
  else if (ws_strcasecmp(month, "Dec") == 0)	gmt.tm_mon = 11;
  gmt.tm_isdst = 0;

  utc_date = mktime(&gmt);

#define	JST_OFFSET	(32400)
  return (utc_date + (time_t)gmt.tm_gmtoff);
}

