/*****************************************************************************
 *
 * FILE:	functions.h
 * DESCRIPTION:	Wyvern: functions declaration file
 * DATE:	Mon, Sep 27 1999
 * UPDATE:	Sat, Aug 25 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *   Copyright (c) 1999-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: functions.h,v 1.17 2007/08/26 16:37:36 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_FUNCTIONS_H
#define	_FUNCTIONS_H

/*
 *	alias.c
 */
extern void	wv_add_aliases(Alias **, const char *, const char *, alias_t);
extern int	wv_alias_translater(Message *);

/*
 *	cond.c
 */
extern int	wv_conditional_request_checker(Message *);

/*
 *	config.c
 */
extern void	wv_init_httpd(void);
extern void	wv_read_config_file(const char *);

/*
 *	date.c
 */
extern time_t	wv_convert_date(const char *);

/*
 *	error.c
 */
extern void	wv_add_error_documents(ErrorDoc **,
				       const int, const char *, errdoc_t);
extern int	wv_error_doc_checker(Message *);
extern int	wv_error_message_sender(Message *);

/*
 *	file.c
 */
extern StrList *	wv_add_indexes(char *);
extern int		wv_document_root_translater(Message *);
extern int		wv_complement_path(Message *);
extern int		wv_open_request_file(Message *);

/*
 *	headers.c
 */
extern int	wv_header_parser(Message *);

/*
 *	hook.c
 */
extern int	wm_run_request_reader(Message *);
extern int	wm_run_header_parser(Message *);
extern int	wm_run_access_checker(Message *);
extern int	wm_run_uri_translater(Message *);
extern int	wm_run_user_id_checker(Message *);
extern int	wm_run_auth_checker(Message *);
extern int	wm_run_type_checker(Message *);
extern int	wm_run_fixups(Message *, hook_ordering_t);
extern int	wm_run_request_handler(Message *);
extern int	wm_run_contents_filter(Message *);
extern int	wm_run_response_handler(Message *);
extern int	wm_run_logger(Message *);
extern void	wm_hook_request_reader(hook_callback_t, hook_ordering_t);
extern void	wm_hook_header_parser(hook_callback_t, hook_ordering_t);
extern void	wm_hook_access_checker(hook_callback_t, hook_ordering_t);
extern void	wm_hook_uri_translater(hook_callback_t, hook_ordering_t);
extern void	wm_hook_user_id_checker(hook_callback_t, hook_ordering_t);
extern void	wm_hook_auth_checker(hook_callback_t, hook_ordering_t);
extern void	wm_hook_type_checker(hook_callback_t, hook_ordering_t);
extern void	wm_hook_fixups(hook_callback_t, hook_ordering_t);
extern void	wm_hook_request_handler(hook_callback_t, hook_ordering_t,
					const char *);
extern void	wm_hook_contents_filter(hook_callback_t, hook_ordering_t,
					const char *);
extern void	wm_hook_response_handler(hook_callback_t, hook_ordering_t);
extern void	wm_hook_logger(hook_callback_t, hook_ordering_t);

/*
 *	log.c
 */
extern void	wv_open_log_files(void);
extern void	wv_close_log_files(void);
extern void	wv_close_log(VirtualHost *);
extern int	wv_log_writer(Message *);
extern void	wv_die(const char *, ...);
extern void	wv_syslog(int, const char *, ...);

/*
 *	mime.c
 */
extern void	wv_add_media_types(const char *, const char *);
extern void	wv_add_types(Hash **, const char *, const char *);
extern void	wv_fixup_mime_table(Hash **);
extern int	wv_media_type_checker(Message *);
#if	DEBUG
extern void	debugMimeTable(Hash *);
#endif	/* DEBUG */

/*
 *	module.c
 */
extern int	wv_load_module(const char *);
extern int	wv_unload_module(const char *);
extern void	wv_initialize_modules(void);
extern void	wv_remove_modules(int);

/*
 *	network.c
 */
extern void	wv_add_sockets(const char *, const char *, const int);
extern void	wv_open_channels(void);
extern void	wv_close_channels(void);

/*
 *	request.c
 */
extern Message *	wv_request_new(SocketInfo *);
extern void		wv_request_clear(Message *, bool);
extern int		wv_request_engine(Message *);
extern int		wv_request_reader(Message *);
extern int		wv_parse_uri(Message *);
extern int		wv_request_handler(Message *);

/*
 *	response.c
 */
extern int	wv_response_engine(Message *);
extern int	wv_response_handler(Message *);
extern int	wv_make_response_header(Message *);
extern int	wv_send_response_header(Message *);

/*
 *	security.c
 */
extern void	wv_enforce_secure_level(void);
extern void	wv_drop_privilege(void);
extern int	wv_safe_path_checker(Message *, const char *);

/*
 *	signal.c
 */
extern void	wv_signal_trap(void);

/*
 *	user.c
 */
extern int	wv_user_dir_translater(Message *);
extern void	wv_user_cache_init(void);

/*
 *	vhost.c
 */
extern VirtualHost *	wv_virtual_host_new(void);
extern void		wv_delete_virtual_host(void);
extern int		wv_switch_virtual_host(Message *);

/*
 *	wyvern.c
 */
extern void	wv_quit(void);

#endif	/* _FUNCTIONS_H */
