/*****************************************************************************
 *
 * FILE:	module.h
 * DESCRIPTION:	Wyvern: module manager header file
 * DATE:	Sat, Sep 16 2000
 * UPDATED:	Sat, Aug 18 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2000-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2000-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: module.h,v 1.10 2007/08/19 06:45:31 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_MODULE_H
#define	_MODULE_H

/******************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/

#ifndef	WM_MAGIC_NUMBER_MAJOR
#define	WM_MAGIC_NUMBER_MAJOR	0x20060301
#endif	/* WM_MAGIC_NUMBER_MAJOR */
#define	WM_MAGIC_NUMBER_MINOR	0	/* 0...n */

/* Hook position */
typedef enum {
  HOOK_INITIAL	= -1,	/* run this hook first, before ANYTHING */
  HOOK_FIRST	= 0,	/* run this hook first */
  HOOK_MIDDLE	= 1,	/* run this hook somewhere */
  HOOK_LAST	= 2,	/* run this hook after every other hook */
  HOOK_FINAL	= 3	/* run this hook last, after EVERYTHING */
} hook_ordering_t;
#define	HOOK_FIXUP_URI		HOOK_INITIAL
#define	HOOK_FIXUP_REQUEST	HOOK_FIRST
#define	HOOK_FIXUP_FILTER	HOOK_MIDDLE
#define	HOOK_FIXUP_RESPONSE	HOOK_LAST

#define	WM_ALL_MODULES		(3)	/* bitmask for flags */
#define	WM_UNSECURE_MODULES	(0)

/* wyvern module structure */
typedef struct {
  const int	version;		/* API version */
#define	WM_UNSECURE	(0)
#define	WM_SECURE	(1)
#define	WM_FIXED	(2)		/* disable unload */
  const int	flags;			/* module flags */
  const char *	name;			/* module name */
  const char *	info;			/* description such as version etc.. */
  const char *	author;			/* module creator */
  const char *	depend;			/* depended module name */
  void *	handle;			/* module handle */
  void		(*init)(void);		/* called when it's loaded */
  void		(*finish)(void);	/* called when it's unloaded */
#define	WM_DLSYM_NAME	"wmModuleInfo"
  void		(*register_hooks)(void);
} wmModule;

#define	WM_RET_ERROR	(-1)
#define	WM_RET_OK	(0)
#define	WM_RET_DECLINED	(1)

/******************************************************************************
 *
 *	Global function declaration
 *
 *****************************************************************************/
extern wmModule *	wmModuleInfo(void);

#endif	/* _MODULE_H */
