/*****************************************************************************
 *
 * FILE:	request.c
 * DESCRIPTION:	Wyvern: address the request URI from client
 * DATE:	Sun, Sep 26 1999
 * UPDATED:	Tue, Sep 11 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: request.c,v 1.32 2007/09/12 02:22:15 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <errno.h>
#include "wyvern.h"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

Message *
wv_request_new(sinfo)
	SocketInfo *	sinfo;
{
#define	AARRAY_SIZE	(32)
  Message *	new;

  new = (Message *)ws_calloc(1, sizeof(Message));	/* leak */
  new->soci.secure	    = sinfo->secure;
  new->soci.port	    = sinfo->port;
  new->ka.status	    = KA_DISABLED;
  new->ka.timeout	    = DEF_KA_TIMEOUT;
  new->ka.max		    = DEF_KA_MAX;
  new->request.http.version = HTTP_10;
  new->request.header	    = wa_new(AARRAY_SIZE);
  new->response.header	    = wa_new(AARRAY_SIZE);
  new->response.status	    = STATUS_NONE;
  new->vhost		    = httpd->host;

  return new;
}

void
wv_request_clear(msg, f_all)
	register Message *	msg;
	bool			f_all;	/* true: all clear, false: partial */
{
  register int	i;

  if (f_all) {
    wv_free(msg->client.ip);
    wv_free(msg->client.port);
    wv_free(msg->client.name);
    msg->ka.status	= KA_DISABLED;
    msg->ka.timeout	= DEF_KA_TIMEOUT;
    msg->ka.max		= DEF_KA_MAX;
    msg->vhost		= httpd->host;
  }
  httpd->vhost		= httpd->host;
  /* turn around time */
  msg->time.request.tv_sec   = 0;
  msg->time.request.tv_usec  = 0;
  msg->time.response.tv_sec  = 0;
  msg->time.response.tv_usec = 0;
  /* request */
  msg->request.method.type = M_NONE;
  wv_free(msg->request.method.name);
  memset(msg->request.uri, 0, sizeof(msg->request.uri));
  msg->request.http.version = HTTP_10;
  wv_free(msg->request.http.protocol);
  wa_flush(msg->request.header);
  msg->request.content_length = 0;
  wv_free(msg->request.auth_type);
  wv_free(msg->request.path_info);
  wv_free(msg->request.query_string);
  if (msg->request.post_fd > 0) {
    close(msg->request.post_fd);
    msg->request.post_fd = -1;
  }
  wv_free(msg->request.post_tmp);
  /* response */
  msg->response.status     = STATUS_NONE;
  msg->response.error_mesg = NULL;
  wa_flush(msg->response.header);
  msg->response.content_length	= 0;
  /* content */
  if (msg->file.nfd != NULL) {
    msg->close(msg->file.nfd);
    msg->file.nfd = NULL;
  }
  msg->file.offset	= 0;
  wv_free(msg->file.path);
  msg->file.mime	= NULL;
  memset((char *)&msg->file.status, 0, sizeof(struct stat));
  wv_free(msg->file.ranges);
  msg->file.ritems	= 0;
  wv_free(msg->file.boundary);
  /* user */
  wv_free(msg->user.indexDir);
  wv_free(msg->user.name);
  msg->user.homeDir = NULL;
  wv_free(msg->user.authName);
  wv_free(msg->user.identName);
  /* cgi */
  msg->cgi.pid	= -1;
  if (msg->cgi.env != NULL) {
    for (i = 0; i < msg->cgi.envno; i++) {
      wv_free(msg->cgi.env[i]);
    }
    wv_free(msg->cgi.env);
  }
}

/*****************************************************************************/

int
wv_request_engine(msg)
	register Message *	msg;
{
  int	status = -1;

  errno = 0;
  if (wm_run_request_reader(msg) == -1 ||	/* Request Reading */
      wm_run_header_parser(msg)  == -1) {	/* Header Parsing */
    if (errno == ETIMEDOUT || errno == ECONNRESET) {
      msg->response.status = STATUS_REQUEST_TIMEOUT;
    }
    return status;
  }

  /* Access Checking */
  if (wm_run_access_checker(msg) == -1) {
    if (msg->client.name == NULL) {
      wv_syslog(WLL_LOG_SECURITY, "denied access from %s", msg->client.ip);
    }
    else {
      wv_syslog(WLL_LOG_SECURITY, "denied access from %s(%s)",
		msg->client.name, msg->client.ip);
    }
    msg->response.status = STATUS_FORBIDDEN;
    return status;
  }

  if (wm_run_fixups(msg, HOOK_FIXUP_URI) == -1 ||
      wm_run_uri_translater(msg)         == -1 ||	/* URI Translating */
      wm_run_user_id_checker(msg)        == -1 ||
      wm_run_auth_checker(msg)           == -1) {	/* Authenticating */
    return status;
  }

  /* Processing */
  if (wm_run_type_checker(msg) == 0 &&	/* MIME Checking */
      wm_run_fixups(msg, HOOK_FIXUP_REQUEST) == 0) {
    if (wm_run_request_handler(msg) != WM_RET_ERROR) {
      /* Filtering */
      if (wm_run_fixups(msg, HOOK_FIXUP_FILTER) == 0) {
	DBG(fprintf(stderr, "DEBUG: Filter %s\n", msg->file.mime);)
  	wm_run_contents_filter(msg);
      }
    }
  }
  errno	 = 0;
  status = wv_error_doc_checker(msg);

  return status;
}

/*
 * Function Name:	wv_request_reader
 * Description:		receive and parse request line
 * Arguments:		message
 * Return Value:	successful: 0, otherwise: -1
 */
int
wv_request_reader(msg)
	register Message *	msg;
{
#define	HTTP_CHECK	"HTTP/"
#define	HTTP_METHOD	(0)
#define	HTTP_URI	(1)
#define	HTTP_VERSION	(2)
#define	MAX_TOKEN	(3)
  char			buf[BUF_SIZE];
  register int		n;
  register char **	ap;
  char *		args[MAX_TOKEN];
  char *		p;

  /* Timestamp of Request */
  gettimeofday(&msg->time.request, (struct timezone *)0);

  /* Read the HTTP request line */
  memset(buf, 0, sizeof(buf));
  n = msg->readline(msg->nsd, buf, sizeof(buf), WT_UTIME_SEC(msg->ka.timeout));
#if	1
  switch (n) {
    case 0:	/* XXX */
      if (msg->ka.status == KA_CONTINUE) { errno = EAGAIN; }
    case -1:
      return WM_RET_ERROR;
    default:
      break;
  }
#else
  if (n <= 0) {
    return WM_RET_ERROR;
  }
#endif

  /* Divide the request-line into the method, URI and protocol */
  for (n = 0, p = buf, ap = args; (*ap = wu_strsep1(&p, " \r\n")) != NULL; ) {
    if (**ap != EOL) {
      n++;
      if (++ap >= &args[MAX_TOKEN]) {
	break;
      }
    }
  }
  /* Invalid request-line */
  if (n != MAX_TOKEN) {
    msg->response.status = STATUS_BAD_REQUEST;
    return WM_RET_ERROR;
  }

  /* HTTP Protocol */
  msg->request.http.protocol = ws_strdup(args[HTTP_VERSION]);

  /* Method */
  msg->request.method.name = ws_strdup(args[HTTP_METHOD]);
  switch (args[HTTP_METHOD][0]) {
    case 'H':	/* HEAD */
      msg->request.method.type	= M_HEAD;
      break;
    case 'G':	/* GET */
      msg->request.method.type	= M_GET;
      break;
    case 'P':	/* POST */
      msg->request.method.type	= M_POST;
      break;
    default:
      msg->request.method.type	= M_UNKNOWN;
      msg->response.status	= STATUS_NOT_IMPLEMENTED;
      return WM_RET_ERROR;
  }

  /* Request-URI */
  if (strlcpy(msg->request.uri, args[HTTP_URI],
	      sizeof(msg->request.uri)) >= sizeof(msg->request.uri)) { 
    msg->response.status = STATUS_REQUEST_URI_TOO_LONG;
    return WM_RET_ERROR;
  }

  /* HTTP-Version */
  if (ws_strncmp(args[HTTP_VERSION], HTTP_CHECK, 5) != 0) {
    msg->response.status = STATUS_BAD_REQUEST;
    return WM_RET_ERROR;
  }
  switch (args[HTTP_VERSION][7]) {
    case '9':	/* HTTP/0.9 */
      msg->request.http.version	= HTTP_09;
      break;
    case '0':	/* HTTP/1.0 */
      msg->request.http.version	= HTTP_10;
      break;
    case '1':	/* HTTP/1.1 */
      msg->request.http.version	= HTTP_11;
      msg->ka.status		= KA_CONTINUE;
      break;
    default:
      msg->request.http.version	= HTTP_00;
      msg->response.status	= STATUS_HTTP_VERSION_NOT_SUPPORTED;
      return WM_RET_ERROR;
  }

  return WM_RET_OK;
}

/*
 * Function Name:	wv_parse_uri
 * Description:		parse request uri
 * Arguments:		message
 * Return Value:	successful: 0, otherwise: -1
 */
int
wv_parse_uri(msg)
	register Message *	msg;
{
  register char *	urip;

  /* Set a query string */
  for (urip = msg->request.uri; *urip != EOL && *urip != '?'; urip++) {
    ;	/* nothing to do */
  }
  if (*urip == '?') {
    *urip++ = EOL;	/* overwrite '?' to EOL */
    msg->request.query_string = ws_strdup(wu_chomp(urip));
  }

  /* Parse a percent-encoding URI (URI length is short after decoding) */
  urip = wu_uri_unescaped(msg->request.uri);
  if (urip == NULL) {
    msg->response.status = STATUS_INTERNAL_SERVER_ERROR;
    return WM_RET_ERROR;
  }
  strlcpy(msg->request.uri, urip, sizeof(msg->request.uri));
  free(urip);

  /* Empty URI */
  if (msg->request.uri[0] == EOL) {
    msg->response.status = STATUS_BAD_REQUEST;
    return WM_RET_ERROR;
  }

  /* Forbid a request URI with relative path */
  urip = (char *)strstr(msg->request.uri, "../");
  if (urip != NULL) {
    msg->response.status = STATUS_FORBIDDEN;
    return WM_RET_ERROR;
  }

  /* Authentication files */
  urip = strrchr(msg->request.uri, '/');
  if (urip++ != NULL) {
    if (ws_strcmp(urip, WYVERN_USER_AUTH)   == 0 ||
	ws_strcmp(urip, WYVERN_USER_PASSWD) == 0 ||
	ws_strcmp(urip, WYVERN_USER_DIGEST) == 0) {
      msg->response.status = STATUS_FORBIDDEN;
      return WM_RET_ERROR;
    }
  }

  return WM_RET_OK;
}

/*
 * Function Name:	wv_request_handler
 * Description:		default request handler
 * Arguments:		message
 * Return Value:	successful: 0, otherwise: -1
 */
int
wv_request_handler(msg)
	register Message *	msg;
{
  if (ws_strcmp(msg->file.mime, MAGIC_TYPE_CGI)	   == 0 ||
      ws_strcmp(msg->file.mime, MAGIC_TYPE_ECGI)   == 0 ||
      ws_strcmp(msg->file.mime, MAGIC_TYPE_SUEXEC) == 0 ||
      ws_strcmp(msg->file.mime, MAGIC_TYPE_DIR)	   == 0) {
    msg->response.status = STATUS_FORBIDDEN;
    return WM_RET_ERROR;
  }

  if (wv_open_request_file(msg) == -1 ||	/* Open the request file */
      /* Check conditional request */
      wv_conditional_request_checker(msg) == -1) {
    return WM_RET_ERROR;
  }
  msg->response.status = STATUS_REQUEST_OK;

  return WM_RET_OK;
}
