/*****************************************************************************
 *
 * FILE:	signal.c
 * DESCRIPTION:	Wyvern: signal handler
 * DATE:	Mon, Nov 15 1999
 * UPDATED:	Mon, Feb 20 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: signal.c,v 1.18 2006/02/19 21:52:46 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <signal.h>
#if	SIGTSTP
#if	HAVE_SYS_FILE_H
#include <sys/file.h>
#endif	/* HAVE_SYS_FILE_H */
#if	HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif	/* HAVE_SYS_IOCTL_H */
#endif	/* SIGTSTP */
#if	HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif	/* HAVE_SYS_WAIT_H */
#include "wyvern.h"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static void	trap(int);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wv_signal_trap
 * Description:		set the signal handler
 * Arguments:		none
 * Return Value:	none
 */
void
wv_signal_trap(void)
{
  struct sigaction	act;

  /* quit */
  sigemptyset(&act.sa_mask);
  act.sa_flags	 = 0;
  act.sa_handler = (void *)trap;
  sigaction(SIGINT,  &act, NULL);
  sigaction(SIGQUIT, &act, NULL);
  sigaction(SIGTERM, &act, NULL);
  /* bug */
  sigaction(SIGBUS,  &act, NULL);
  sigaction(SIGSEGV, &act, NULL);
  /* restart */
  act.sa_flags	 = SA_RESTART;
  act.sa_handler = (void *)trap;
  sigaction(SIGHUP,  &act, NULL);
}

/*
 * Function Name:	trap
 * Description:		trap all signals
 * Arguments:		signal
 * Return Value:	never return
 */
static void
trap(signo)
	int	signo;
{
  switch (signo) {
    case SIGHUP:
      httpd->sighup = true;
      return;
    case SIGBUS:
    case SIGSEGV:
      wv_syslog(WLL_LOG_CRIT, "killed by sig#%d .. it's a BUG!", signo);
      break;
    case SIGINT:
    case SIGQUIT:
    case SIGTERM:
      wv_syslog(WLL_LOG_NOTICE, "terminated by sig#%d", signo);
      break;
    default:
      wv_syslog(WLL_LOG_NOTICE, "terminated by unexpected sig#%d", signo);
      break;
  }
  wv_quit();
}

