/*****************************************************************************
 *
 * FILE:	logview.c
 * DESCRIPTION:	output an access.log with decoded escaped URI
 * DATE:	Thu, Apr 20 2000
 * UPDATED:	Wed, Apr 13 2005
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2000-2005 졿Kouichi ABE (WALL), All rights reserved.
 * HOWTOMAKE:	gcc -O2 -I/usr/local/include -L/usr/local/lib logview.c -lwkf
 * LICENSE:
 *
 *  Copyright (c) 2000-2005 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: logview.c,v 1.4 2005/04/13 14:47:16 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	ENABLE_WKF
#include <wkf.h>
#endif	/* ENABLE_WKF */

/*****************************************************************************
 *
 *	Copyright
 *
 *****************************************************************************/
const char	Program[]   = "logview";
const char	Copyright[] =
	"Copyright (c) 2000-2005 Kouichi ABE (WALL), All rights reserved.";

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/* Numeric release version identifier: MNNFFPPS: major minor fix patch status */
#define	LOGVIEW_NAME		"logview"
#define LOGVIEW_VERSION_NUMBER	0x10001010L
#define LOGVIEW_VERSION		"1.0.1 (2005/04/13)"
#define LOGVIEW_VERSION_TEXT	LOGVIEW_NAME "/" LOGVIEW_VERSION

#ifndef	EOL
#define	EOL	'\0'
#endif	/* EOL */

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static char *	decodeEscapedURI(char *);
static void	parser(FILE *);
static void	version(void);
static void	usage(void);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
/*
 * Function Name:	decodeEscapedURI
 * Description:		escaped character is decoded
 * Arguments:		string
 * Return Value:	decoded string
 */
static char *
decodeEscapedURI(s)
	char *	s;
{
  char *		tmp;
  register char *	os;
  register char *	ret;
  char			c, c1;

  tmp = strdup(s);
  os  = tmp;
  ret = s;
  while ((c = *os) != EOL) {
    if (c == '%') {
      os++;
      if ((c = *os++) && (c1 = *os++)) {
	*s++ =	((c  >= 'A' ? ((c  & 0xdf) - 'A') + 10 : c  - '0') * 16) +
		((c1 >= 'A' ? ((c1 & 0xdf) - 'A') + 10 : c1 - '0'));
	os--;	/* back one word */
      }
      else {
        return ret;
      }
    }
    /* escaped white space */
    else if (c == '+') {
      *s++ = ' ';
    }
    /* normal characters */
    else {
      *s++ = c;
    }
    os++;
  }
  *s = EOL;
  free((char *)tmp);

  return ret;
}

/*
 * Function Name:	parser
 * Description:		frontend for decodeEscapedURI
 * Arguments:		file pointer
 * Return Value:	none
 */
static void
parser(fin)
	register FILE *	fin;
{
  char	buf[BUFSIZ];
#if	ENABLE_WKF
  char	ret[BUFSIZ*2];
#endif	/* ENABLE_WKF */

  memset(buf, 0, sizeof(buf));
  while (fgets(buf, sizeof(buf), fin) != NULL) {
#if	ENABLE_WKF
    decodeEscapedURI(buf);
#if	WKF_RELEASE
    if (wkfConvertKanjiCodeOfString(KC_UNKNOWN, buf, KC_EUC, ret, sizeof(ret)) == CONV_OK) {
      fputs(ret, stdout);
    }
#else
    if (wkfConvertKanjiCodeOfString(KC_EUC, buf, ret) == CONV_OK) {
      fputs(ret, stdout);
    }
#endif	/* WKF_RELEASE */
    else {
      fputs(buf, stdout);
    }
    memset(buf, 0, sizeof(buf));
    memset(ret, 0, sizeof(ret));
#else
    fputs(decodeEscapedURI(buf), stdout);
#endif	/* ENABLE_WKF */
  }
}

/*****************************************************************************/

static void
version(void)
{
  fprintf(stderr, "%s\n%s\n\n", LOGVIEW_VERSION_TEXT, Copyright);
  exit(64);
}

static void
usage(void)
{
  fprintf(stderr,
	  "usage: %s [<access_log>]\n",
	  Program);
  exit(64);
}

/*****************************************************************************/

/*
 *	MAIN
 */
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  register int	ch;
  register FILE *	fin;

  while ((ch = getopt(argc, argv, "hv?")) != -1) {
    switch (ch) {
      case 'v': version();
      default:	usage();
    }
  }
  argc -= optind;
  argv += optind;

  if (argc == 0) {
    fin = stdin;
    parser(fin);
  }
  else {
    register int	i;

    for (i = 0; i < argc; i++) {
      fin = fopen(argv[i], "r");
      if (fin != NULL) {
	parser(fin);
	fclose(fin);
      }
    }
  }

  return 0;
}

