#!/bin/sh
##############################################################################
#
# FILE:		mk_rules.sh
# DESCRIPTION:	Wyvern: make PostgreSQL rules for mod_log_pgsql.so
# DATE:		Sat, Aug 25 2007
# UPDATED:	Sun, Aug 26 2007
# AUTHOR:	Kouichi ABE (WALL) / 
# E-MAIL:	kouichi@MagickWorX.COM
# URL:		http://www.MagickWorX.COM/
# COPYRIGHT:	(c) 2007 졿Kouichi ABE (WALL), All rights reserved.
# LICENSE:
#
#  Copyright (c) 2007 Kouichi ABE (WALL) <kouichi@MagickWorX.COM>,
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
#   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
#   THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: mk_rules.sh,v 1.1 2007/08/26 16:37:59 kouichi Exp $
#
##############################################################################

mk_access_log_rules() {
  local	year=$1
  local	month=$2
  local	end_y
  local	end_m
  local	st
  local	ed
  local	name

  if [ ${month} -eq 12 ]; then
    end_y=$((${year} + 1))
    end_m=1
  else
    end_y=${year}
    end_m=$((${month} + 1))
  fi
  st=$(printf "DATE '%d-%02d-01'" ${year} ${month})
  ed=$(printf "DATE '%d-%02d-01'" ${end_y} ${end_m})

  name=$(printf "%d%02d" ${year} ${month})

  cat << __EOT__
CREATE TABLE access_log_${name} (
  check (timestamp >= ${st} AND timestamp < ${ed})
) INHERITS (access_log);
CREATE INDEX access_log_${name}_idx01 ON access_log_${name} (timestamp);

GRANT SELECT,INSERT ON access_log_${name} TO wyvern;

CREATE OR REPLACE RULE access_log_insert_${name} AS
ON INSERT TO access_log WHERE
  (timestamp >= ${st} AND timestamp < ${ed})
DO INSTEAD
  INSERT INTO access_log_${name} VALUES (NEW.hostname, NEW.timestamp, NEW.ipaddr, NEW.forwarded_for, NEW.ident_name, NEW.auth_name, NEW.method, NEW.request_uri, NEW.protocol, NEW.status, NEW.length, NEW.phrase, NEW.referer, NEW.user_agent, NEW.uptime);

__EOT__
}

mk_access_log_all() {
  local	year=$1
  local	m

  for m in 1 2 3 4 5 6 7 8 9 10 11 12;
  do
    mk_access_log_rules ${year} ${m}
  done
}

###############################################################################

mk_access_log_cc_rules() {
  local	year=$1
  local	month=$2
  local	end_y
  local	end_m
  local	st
  local	ed
  local	name

  if [ ${month} -eq 12 ]; then
    end_y=$((${year} + 1))
    end_m=1
  else
    end_y=${year}
    end_m=$((${month} + 1))
  fi
  st=$(printf "DATE '%d-%02d-01'" ${year} ${month})
  ed=$(printf "DATE '%d-%02d-01'" ${end_y} ${end_m})

  name=$(printf "%d%02d" ${year} ${month})

  cat << __EOT__
CREATE TABLE access_log_cc_${name} (
  check (timestamp >= ${st} AND timestamp < ${ed})
) INHERITS (access_log_cc);
CREATE INDEX access_log_cc_${name}_idx01 ON access_log_cc_${name} (timestamp);

GRANT SELECT,INSERT ON access_log_cc_${name} TO wyvern;

CREATE OR REPLACE RULE access_log_cc_insert_${name} AS
ON INSERT TO access_log_cc WHERE
  (timestamp >= ${st} AND timestamp < ${ed})
DO INSTEAD
  INSERT INTO access_log_cc_${name} VALUES (NEW.hostname, NEW.timestamp, NEW.ipaddr, NEW.forwarded_for, NEW.ident_name, NEW.auth_name, NEW.method, NEW.request_uri, NEW.protocol, NEW.status, NEW.length, NEW.phrase, NEW.referer, NEW.user_agent, NEW.country, NEW.region, NEW.uptime);

__EOT__
}

mk_access_log_cc_all() {
  local	year=$1
  local	m

  for m in 1 2 3 4 5 6 7 8 9 10 11 12;
  do
    mk_access_log_cc_rules ${year} ${m}
  done
}

###############################################################################

usage() {
  local	progname=$(basename $0)
  (
    echo "Copyright (c) 2007 Kouichi ABE (WALL), All rights reserved."
    echo "usage: ${progname} <access_log>|<access_log_cc> [<year> [<month>]]"
    echo
  ) 1>&2
  exit 64
}

###############################################################################

# arguments processing
args=`getopt hv $*`
if [ $? -ne 0 ]; then usage; fi
set -- ${args}

for i
do
  case "$i" in
    -h|-v) usage ;;
    --) shift; break ;;
  esac
done

this_year=$(date +%Y)
case "$1" in
  access_log)
    if [ $# -eq 2 ]; then
      mk_access_log_all $2
    elif [ $# -eq 3 ]; then
      mk_access_log_rules $2 $3
    else
      mk_access_log_all ${this_year}
    fi
    ;;
  access_log_cc)
    if [ $# -eq 2 ]; then
      mk_access_log_cc_all $2
    elif [ $# -eq 3 ]; then
      mk_access_log_cc_rules $2 $3
    else
      mk_access_log_cc_all ${this_year}
    fi
    ;;
  *)	  usage ;;
esac

exit 0
