/*****************************************************************************
 *
 * FILE:	sqlite_logview.c
 * DESCRIPTION:	Wyvern: log viewr for mod_log_sqlite.c
 * DATE:	Wed, Apr 13 2005
 * UPDATED:	Wed, Apr 13 2005
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2005 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2005 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sqlite_logview.c,v 1.1 2005/04/13 14:47:16 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	ENABLE_SQLITE3
#include <sqlite3.h>
#elif	ENABLE_SQLITE2
#include <sqlite.h>
#endif	/* ENABLE_SQLITE3 */
#include <wyvern.h>

/*****************************************************************************
 *
 *	Copyright
 *
 *****************************************************************************/
const char	Program[]   = "sqlite_logview";
const char	Copyright[] =
	"Copyright (c) 2005 Kouichi ABE (WALL), All rights reserved.";

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/* Numeric release version identifier: MNNFFPPS: major minor fix patch status */
#define	LOGVIEW_NAME		"sqlite_logview"
#define LOGVIEW_VERSION_NUMBER	0x10000010L
#define LOGVIEW_VERSION		"1.0 (2005/04/13)"
#define LOGVIEW_VERSION_TEXT	LOGVIEW_NAME "/" LOGVIEW_VERSION

#if	DEBUG
#define	SQLITE_LOG_DB	"/tmp/www/sqlite_log.db"
#endif	/* DEBUG */

#ifndef	SQLITE_LOG_DB
#define	SQLITE_LOG_DB	WYVERN_DB_DIR "/sqlite_log.db"
#endif	/* SQLITE_LOG_DB */

#define	ACCESS_LOG	"access_log"
#define	ERROR_LOG	"error_log"
#define	VIEW_ACCESS_LOG	(1)
#define	VIEW_ERROR_LOG	(2)

#define	SELECT_ACCESS	"SELECT * FROM access_log;"
#define	SELECT_ERROR	"SELECT * FROM error_log;"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	logview(int, const char *);
static void	version(void);
static void	usage(void);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
#if	ENABLE_SQLITE3
static int
logview(type, fname)
	int		type;	/* log type */
	const char *	fname;	/* database name */
{
  sqlite3 *		sdb;
  sqlite3_stmt *	stmt_view;
  int			rc;

  rc = sqlite3_open(fname, &sdb);
  if (rc != SQLITE_OK) {
    return -1;
  }

  switch (type) {
    case VIEW_ACCESS_LOG:
      rc = sqlite3_prepare(sdb, SELECT_ACCESS, (int)strlen(SELECT_ACCESS),
			   &stmt_view, NULL);
      break;
    case VIEW_ERROR_LOG:
      rc = sqlite3_prepare(sdb, SELECT_ERROR, (int)strlen(SELECT_ERROR),
			   &stmt_view, NULL);
      break;
    default:
      break;
  }
  if (rc == SQLITE_OK) {
    do {
      rc = sqlite3_step(stmt_view);
      switch (rc) {
	case SQLITE_ROW: {
	    int	n;

	    n = sqlite3_data_count(stmt_view);
	    if (n > 0) {
	      register int	i;

	      for (i = 0; i < n; i++) {
		char *	p;

		p = sqlite3_column_text(stmt_view, i);
		if (p != NULL) {
		  fprintf(stdout, " %s", p);
		}
	      }
	      fprintf(stdout, "\n");
	    }
	  }
	  break;
	case SQLITE_DONE:
	default:
	  break;
      }
    } while (rc == SQLITE_ROW);
    sqlite3_finalize(stmt_view);
  }
  sqlite3_close(sdb);

  return 0;
}
#elif	ENABLE_SQLITE2
static int
logview(type, fname)
	int		type;	/* log type */
	const char *	fname;	/* database name */
{
  sqlite *	sdb;

  sdb = sqlite_open(fname, 0, NULL);
  if (sdb != NULL) {
    return -1;
  }

  switch (type) {
    case VIEW_ACCESS_LOG: {
	int		rc;
	sqlite3_stmt *	stmt_view;

	rc = sqlite3_prepare(sdb, SELECT_ACCESS, (int)strlen(SELECT_ACCESS),
			     &stmt_view, NULL);
	if (rc == SQLITE_OK) {
	  do {
	    rc = sqlite3_step(stmt_view);
	    switch (rc) {
	      case SQLITE_ROW: {
		  int	n;

		  n = sqlite3_data_count(stmt_view);
		  if (n > 0) {
		    register int	i;
		  }
		}
		break;
	      case SQLITE_DONE:
	      default:
		break;
	    }
	  } while (rc == SQLITE_ROW);
	  sqlite3_finalize(stmt_view);
	}
      }
      break;
    case VIEW_ERROR_LOG: {
      }
      break;
    default:
      break;
  }
  sqlite_close(sdb);

  return 0;
}
#endif	/* ENABLE_SQLITE3 */

/*****************************************************************************/

static void
version(void)
{
  fprintf(stderr, "%s\n%s\n\n", LOGVIEW_VERSION_TEXT, Copyright);
  exit(64);
}

static void
usage(void)
{
  fprintf(stderr,
	  "usage: %s [-f <sqlite_log.db>] <error_log>|<access_log>\n",
	  Program);
  exit(64);
}

/*****************************************************************************/

/*
 *	MAIN FUNCTION
 */
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  register int	ch;
  const char *	fname = SQLITE_LOG_DB;

  while ((ch = getopt(argc, argv, "f:hv?")) != -1) {
    switch (ch) {
      case 'f': fname = optarg; break;
      case 'v': version();
      default:	usage();
    }
  }
  argc -= optind;
  argv += optind;

  if (argc != 1) {
    usage();
  }
  else if (strcmp(argv[0], ACCESS_LOG) == 0) {
    logview(VIEW_ACCESS_LOG, fname);
  }
  else if (strcmp(argv[0], ERROR_LOG) == 0) {
    logview(VIEW_ERROR_LOG, fname);
  }

  return 0;
}

