/*****************************************************************************
 *
 * FILE:	wpasswd.c
 * DESCRIPTION:	Wyvern: the front end program for 'wpasswd'
 * DATE:	Sat, Dec 12 1999
 * UPDATE:	Tue, Feb 14 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 1999-2006 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 1999-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wpasswd.c,v 1.7 2006/02/14 13:57:29 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#if	HAVE_DB_H
#include <db.h>
#endif	/* HAVE_DB_H */
#if	HAVE_NDBM_H
#include <ndbm.h>
#endif	/* HAVE_NDBM_H */
#if	HAVE_MD5_H
#include <md5.h>
#endif	/* HAVE_MD5_H */
#include <sys/stat.h>
#if	HAVE_TERMIOS_H
#include <termios.h>
#endif	/* HAVE_TERMIOS_H */
#if	ENABLE_SQLITE3
#include <sqlite3.h>
#endif	/* ENABLE_SQLITE3 */
#include <errno.h>
#include "ws.h"
#include "wu.h"

/*****************************************************************************
 *
 *	Copyright
 *
 *****************************************************************************/
const char	Program[]   = "wpasswd";
const char	PrograMD5[] = "wpasswd_md5";
const char	Copyright[] =
	"Copyright (c) 1999-2006 Kouichi ABE (WALL), All rights reserved.";

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/* Numeric release version identifier: MNNFFPPS: major minor fix patch status */
#define	WPASSWD_NAME		"wpasswd"
#define WPASSWD_VERSION_NUMBER	0x10000020L
#define WPASSWD_VERSION		"1.0.0 (2006/02/14)"
#define WPASSWD_VERSION_TEXT	WPASSWD_NAME "/" WPASSWD_VERSION

#ifndef	_BOOL_T
#define	_BOOL_T
typedef enum {	/* boolean type */
  false = 0,
  true  = 1
} bool;
#endif	/* _BOOL_T */

#ifndef	MD5_DIGEST_LENGTH
#define	MD5_DIGEST_LENGTH	(16)
#endif	/* MD5_DIGEST_LENGTH */

#define	EOL	'\0'
#define CR	'\015'
#define LF	'\012'

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static char *	to_MD5(char *);
static char *	get_password(void);
static int	password_with_db(const char *, const char *);
static int	password_with_dbm(const char *, const char *);
#if	ENABLE_SQLITE3
static int	password_with_sqlite3(const char *, const char *);
#endif	/* ENABLE_SQLITE3 */
static void	usage(void);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
static bool	enable_md5 = false;	/* MD5 encription */

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	to_MD5
 * DESCRIPTION:		make a new password
 * ARGUMENTS:		password (plain text)
 * RETURN VALUE:	a crypted password
 */
static char *
to_MD5(text)
	char *	text;
{
  MD5_CTX	md5_ctx;
  unsigned char	md5[MD5_DIGEST_LENGTH];
  unsigned char	md5_hex[MD5_DIGEST_LENGTH*2+1];
  int		i;

  MD5Init(&md5_ctx);
  MD5Update(&md5_ctx, text, strlen(text));
  MD5Final(md5, &md5_ctx);

  memset(md5_hex, 0, sizeof(md5_hex));
  for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
    (void)snprintf(md5_hex + i * 2, sizeof(md5_hex) - 1, "%02x", md5[i]);
  }

  return ws_strdup(md5_hex);
}


/*
 * FUNCTION NAME:	get_password
 * DESCRIPTION:		recept user and password
 * ARGUMENTS:		none
 * RETURN VALUE:	password string with plain text
 */
static char *
get_password(void)
{
  struct termios	tc_old;
  struct termios	tc_new;

  tcgetattr(fileno(stdin), &tc_old);
  for (;;) {
    char	pwd1[BUFSIZ];
    char	pwd2[BUFSIZ];
    size_t	len;

    memset(pwd1, 0, sizeof(pwd1));
    fputs("Password: ", stdout);
    tc_new	    = tc_old;
    tc_new.c_lflag &= ~ECHO;
    tcsetattr(fileno(stdin), TCSAFLUSH, &tc_new);
    if (fgets(pwd1, sizeof(pwd1), stdin) == NULL) {
      exit(0);
    }
    putchar(LF);

    memset(pwd2, 0, sizeof(pwd2));
    fputs("Retype password: ", stdout);
    if (fgets(pwd2, sizeof(pwd2), stdin) == NULL) {
      exit(0);
    }
    putchar(LF);
    tcsetattr(fileno(stdin), TCSANOW, &tc_old);

    len = strlen(pwd1);
    if (len == 1) {
      fputs("Empty Password; try again\n", stderr);
    }
    else if (ws_strncmp(pwd1, pwd2, len) == 0) {
      return ws_strdup(wu_chomp(pwd1));
    }
    else {
      fputs("Mismatch; try again\n", stderr);
    }
  }
}

/*
 * FUNCTION NAME:	password_with_db
 * DESCRIPTION:		register username and password
 * ARGUMENTS:		password file, username
 * RETURN VALUE:	successful: 0, otherwise: -1
 */
static int
password_with_db(password_file, username)
	const char *	password_file;
	const char *	username;
{
  char *	password;
  register DB *	dp;
  DBT		key;
  DBT		val;
  int		ret;

  password = get_password();
  if (enable_md5) {
    password = to_MD5(password);
  }
  if (password_file == NULL || username == NULL || password == NULL) {
    return -1;
  }

  dp = dbopen(password_file, O_RDWR|O_CREAT, 0600, DB_HASH, NULL);
  if (dp == NULL) {
    fprintf(stderr, "ERROR: failed to open %s\n", password_file);
    return -1;
  }

  key.data = (char *)username;
  key.size = strlen(username);
  val.data = password;
  val.size = strlen(password);

  ret = dp->put(dp, &key, &val, 0);

  dp->close(dp);

  return ret;
}

/*
 * FUNCTION NAME:	password_with_dbm
 * DESCRIPTION:		register username and password
 * ARGUMENTS:		password file, username
 * RETURN VALUE:	successful: 0, otherwise: -1
 */
static int
password_with_dbm(password_file, username)
	const char *	password_file;
	const char *	username;
{
  char *		password;
  register DBM *	dp;
  datum			key;
  datum			val;
  int			ret;

  password = get_password();
  if (enable_md5) {
    password = to_MD5(password);
  }
  if (password_file == NULL || username == NULL || password == NULL) {
    return -1;
  }

  dp = dbm_open(password_file, O_RDWR|O_CREAT, 0600);
  if (dp == NULL) {
    fprintf(stderr, "ERROR: failed to open %s\n", password_file);
    return -1;
  }

  key.dptr  = (char *)username;
  key.dsize = strlen(username);
  val.dptr  = password;
  val.dsize = strlen(password);

  ret = dbm_store(dp, key, val, DBM_REPLACE);

  dbm_close(dp);

  return ret;
}

/*
 * FUNCTION NAME:	password_with_sqlite
 * DESCRIPTION:		register username and password
 * ARGUMENTS:		password file, username
 * RETURN VALUE:	successful: 0, otherwise: -1
 */
#if	ENABLE_SQLITE3
#define	SQL_QUERY	"INSERT OR REPLACE INTO wyvern_auth VALUES('%q','%q');"
static int
password_with_sqlite3(password_file, username)
	const char *	password_file;
	const char *	username;
{
  char *	password;
  sqlite3 *	sdb;
  int		rc;

  password = get_password();
  if (enable_md5) {
    password = to_MD5(password);
  }
  if (password_file == NULL || username == NULL || password == NULL) {
    return -1;
  }

  rc = sqlite3_open(password_file, &sdb);
  if (rc == SQLITE_OK) {
    char *	sql;
    char *	emsg = NULL;

    sql = sqlite3_mprintf(SQL_QUERY, username, password);
    if (sql != NULL) {
      rc = sqlite3_exec(sdb, sql, NULL, NULL, &emsg);
      sqlite3_free(sql);
    }
    sqlite3_close(sdb);
    if (rc != SQLITE_OK) {
      fprintf(stderr, "%s\n", emsg);
      sqlite3_free(emsg);
      return -1;
    }
  }

  ws_chmod(password_file, 0600);

  return 0;
}
#endif	/* ENABLE_SQLITE3 */

/*
 * FUNCTION NAME:	usage
 * DESCRIPTION:		print usage
 * ARGUMENTS:		none
 * RETURN VALUE:	none
 */
static void
usage(void)
{
  fprintf(stderr,
	  WPASSWD_VERSION_TEXT "\n"
	  "%s\n"
	  "usage: %s <type> passwordfile username\n"
#if	ENABLE_SQLITE3
	  "          <type> := [ db | dbm | sqlite3 ]\n",
#else
	  "          <type> := [ db | dbm ]\n",
#endif	/* ENABLE_SQLITE3 */
	  Copyright, Program);
  exit(64);
}


/*
 *	MAIN FUNCTION
 */
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  /* check arguments */
  if (argc != 4) { usage(); }

  /* check program name */
  if (ws_strcmp(argv[0], PrograMD5) == 0) {
    enable_md5 = true;
  }

  /* check database type */
  if (ws_strcmp(argv[1], "db") == 0) {
    password_with_db(argv[2], argv[3]);
  }
  else if (ws_strcmp(argv[1], "dbm") == 0) {
    password_with_dbm(argv[2], argv[3]);
  }
#if	ENABLE_SQLITE3
  else if (ws_strcmp(argv[1], "sqlite3") == 0) {
    password_with_sqlite3(argv[2], argv[3]);
  }
#endif	/* ENABLE_SQLITE3 */
  else {
    usage();
  }

  return 0;
}

